from .proto import (
    Permission,
    LeafCrudPermission,
    RetrieveAuditLogPermission,
    ExecuteComputePermission,
    RetrieveDataRoomPermission,
    RetrieveDataRoomStatusPermission,
    UpdateDataRoomStatusPermission,
    RetrievePublishedDatasetsPermission,
    DryRunPermission,
)

class Permissions:
    """Helper class for creating data room permissions."""

    def __init__(self):
        """This class is not meant to be instantiated."""

    @staticmethod
    def leaf_crud(leaf_node_name: str) -> Permission:
        """Permission required for publishing a dataset to a data room."""
        return Permission(
            leafCrudPermission=LeafCrudPermission(leafNodeName=leaf_node_name)
        )

    @staticmethod
    def retrieve_data_room() -> Permission:
        """Permission required for retrieving a data room's definition after it has
        been published."""
        return Permission(retrieveDataRoomPermission=RetrieveDataRoomPermission())

    @staticmethod
    def retrieve_audit_log() -> Permission:
        """Permission for retrieving the audit log, a log detailing all past interactions
        with the data room."""
        return Permission(retrieveAuditLogPermission=RetrieveAuditLogPermission())

    @staticmethod
    def execute_compute(compute_node_name: str) -> Permission:
        """Permission for executing the computation with the given name."""
        return Permission(
            executeComputePermission=ExecuteComputePermission(
                computeNodeName=compute_node_name
            )
        )

    @staticmethod
    def retrieve_data_room_status() -> Permission:
        """Permission for retrieving the status of a data room."""
        return Permission(
            retrieveDataRoomStatusPermission=RetrieveDataRoomStatusPermission()
        )

    @staticmethod
    def update_data_room_status() -> Permission:
        """Permission for updating the status of a data room (e.g. irreversibly stopping it)."""
        return Permission(
            updateDataRoomStatusPermission=UpdateDataRoomStatusPermission()
        )

    @staticmethod
    def retrieve_published_datasets() -> Permission:
        """Permission for retrieving the list of datasets that has been published to the data room."""
        return Permission(
            retrievePublishedDatasetsPermission=RetrievePublishedDatasetsPermission()
        )

    @staticmethod
    def dry_run() -> Permission:
        """Permission for triggering a dry run on the data room."""
        return Permission(dryRunPermission=DryRunPermission())



from .proto import AttestationSpecification
from typing import List, Dict
from typing_extensions import TypedDict
from enum import Enum


class JobId:
    """
    Class for identifying running or already run jobs.

    Objects of this class can be used to retrieve results for processed computations.
    """
    def __init__(self, job_id: str, compute_node_name: str):
        self.id = job_id
        """The identifier of the job that processed a particular computation."""

        self.compute_node_name = compute_node_name
        """The name of the computation that was processed."""


class ScopeTypes(str, Enum):
    USER_FILE = "user_file",
    DATA_ROOM_DEFINITION = "dataroom_definition",
    DATA_ROOM_INTERMEDIATE_DATA = "dataroom_intermediate_data"


class UserResponse(TypedDict):
    id: str
    email: str


class UserCsrRequest(TypedDict):
    csrPem: str


class UserCsrResponse(TypedDict):
    certChainPem: str


class SystemCaResponse(TypedDict):
    rootCertificate: str


class CreateSessionRequest(TypedDict):
    attestationSpecificationHash: str


class SessionJsonResponse(TypedDict):
    sessionId: str
    attestationSpecificationHash: str


class FinalizeUpload(TypedDict):
    uploadId: str
    manifest: str
    name: str
    manifestHash: str
    chunks: List[str]


class ChunkWrapper(TypedDict):
    hash: str
    data: str


class UploadDescription(TypedDict):
    uploadId: str


class ChunkDescription(TypedDict):
    chunkHash: str


class DataRoomDescription(TypedDict):
    dataRoomId: str
    name: str
    description: str
    mrenclave: str
    ownerEmail: str
    status: str


class DatasetDescription(TypedDict):
    """
    This class includes information about an uploaded dataset
    """
    manifestHash: str
    """"""
    filename: str
    """"""
    description: str
    ownerEmail: str
    chunks: List[ChunkDescription]


class SignatureResponse(TypedDict):
    type: str
    data: List[int]


class EnclaveMessage(TypedDict):
    data: str


class FatquoteResBody(TypedDict):
    fatquoteBase64: str


class DatasetManifestMetadata(TypedDict):
    name: str
    manifestHash: str
    chunks: List[str]


class EnclaveSpecification(TypedDict):
    """
    This class includes information about an enclave deployed in the platform.
    Please refer to `decentriq_platform.EnclaveSpecifications` for a detailed explaination.
    """
    name: str
    """The name of the enclave."""
    version: str
    """The version of the enclave."""
    proto: AttestationSpecification
    """The Protobuf object."""


class CreateScopeRequest(TypedDict):
    metadata: Dict[str, str]


class ScopeJson(TypedDict):
    scopeId: str
    metadata: Dict[str, str]


class EnclaveSpecificationJson(TypedDict):
    name: str
    version: str
    spec: str


class EnclaveSpecificationResponse(TypedDict):
    attestationSpecs: List[EnclaveSpecificationJson]


class Tcb(TypedDict):
    sgxtcbcomp01svn: int
    sgxtcbcomp02svn: int
    sgxtcbcomp03svn: int
    sgxtcbcomp04svn: int
    sgxtcbcomp05svn: int
    sgxtcbcomp06svn: int
    sgxtcbcomp07svn: int
    sgxtcbcomp08svn: int
    sgxtcbcomp09svn: int
    sgxtcbcomp10svn: int
    sgxtcbcomp11svn: int
    sgxtcbcomp12svn: int
    sgxtcbcomp13svn: int
    sgxtcbcomp14svn: int
    sgxtcbcomp15svn: int
    sgxtcbcomp16svn: int


class TcbLevel(TypedDict):
    tcb: Tcb
    tcbStatus: str


class TcbInfo(TypedDict):
    version: int
    issueDate: str
    nextUpdate: str
    fmspc: str
    pceId: str
    tcbType: int
    tcbEvaluationDataNumber: int
    tcbLevels: List[TcbLevel]


class TcbInfoContainer(TypedDict):
    tcbInfo: TcbInfo
    signature: str


class IasResponse(TypedDict):
    isvEnclaveQuoteBody: str
    isvEnclaveQuoteStatus: str



import zipfile
import io
import csv
from typing import List, Tuple, Any
from .compute import SqlSchemaVerifier
from .. import Permissions, Session, Key, DataRoomBuilder
from ..compute import Noop
from .proto import TableSchema
from ..proto import Permission
from ..proto.length_delimited import parse_length_delimited


class ValidatedDataNode:
    """
    Helper class to construct the triplet of nodes required for data nodes with schema validation.

    In a data clean room computations and the data they depend on are expressed in terms
    of graphs where computations are represented by *compute nodes* and input datasets are
    represented by *data nodes*.
    With tabular input data, it is possible to validate the input data and check whether it
    corresponds to a pre-defined schema. This validation is a computation in itself so in
    order to validate the input data, we need to add an additional compute node on top of the
    data node. Because it must be possible to trigger this validation *while restricting access
    to the resulting data*, another special node, the *noop node* needs to be added.

    This class will add the necessary nodes to your data clean room.

    After having constructed an object of this class, pass its `add_to_builder` method to
    decentriq_platform.DataRoomBuilder.add_using_function` or pass the builder object to
    `add_to_builder` directly.
    When creating the data clean room, don't forget to also grant the `validation_permission`
    permission as well as the `leaf_crud_permission`. Both permissions should usually be granted
    at the same time.

    Data should be published to the node named `input_node_name` and be read from `output_node_name`.
    The name of the validation computation can be accessed using the field `validation_computation_name`.

    The helper function `upload_and_publish_dataset` will upload, publish, and validate your data
    automatically.
    """
    def __init__(
            self,
            table_name: str,
            schema: List[Tuple[str, Any, bool]],
            *,
            is_required: bool = False
        ):
        """
        Create a `ValidatedDataNode`.

        **Parameters**:
        - `table_name`: What your dataset should be called. This is the name you will later use in
            your SQL queries.
        - `schema`: The list of columns. This is a list of tuples, each containing the name of the column,
            the data type (`decentriq_compute_sql.PrimitiveType`), and whether the column is nullable (can have empty values).
            The data type is an enum like object with values `PrimitiveType.STRING`, `PrimitiveType.INT64`,
            `PrimitiveType.FLOAT64`.
        - `is_required`: Whether the dataset needs to be present for computations to be triggered.
        """
        self.is_required = is_required
        self._leaf_node_name = f"@table/${table_name}/dataset"
        self._verifier_node_name = table_name

        self.validation_computation_name = f"@table/${table_name}/validation"
        """The name of the computation that will perform the data validation."""

        self._verifier = SqlSchemaVerifier(schema)

    def add_to_builder(self, builder: DataRoomBuilder):
        """
        Configure the given `DataRoomBuilder` to build te final data clean room with the necessary
        compute and data nodes.

        Remember that you still have to add the `validation_permission` and `leaf_crud_permission`
        in order for a user to be able to upload and validate the input data.
        """
        builder.add_data_node(self._leaf_node_name, is_required=self.is_required)
        builder.add_compute_node(self._verifier_node_name, self._verifier, dependencies=[self._leaf_node_name])
        builder.add_compute_node(self.validation_computation_name, Noop(), dependencies=[self._verifier_node_name])

    @property
    def validation_permission(self) -> Permission:
        """The permission required to trigger the data validation."""
        return Permissions.execute_compute(self.validation_computation_name)

    @property
    def leaf_crud_permission(self) -> Permission:
        """The permission required to upload the raw, non-validated data."""
        return Permissions.leaf_crud(self._leaf_node_name)

    @property
    def input_node_name(self) -> str:
        """The node name to which data should be uploaded."""
        return self._leaf_node_name

    @property
    def output_node_name(self) -> str:
        """The node name from which data can be read, e.g. the name to use in any downstream SQL queries."""
        return self._verifier_node_name


def read_input_csv_file(
        path: str,
        /, *,
        has_header: bool = True,
        check_header: bool = True,
        encoding="utf-8",
        **kwargs
) -> io.BytesIO:
    """
    Read CSV from a file and turn it into a bytes array of the correct format so that it can be uploaded
    to the Decentriq platform.

    **Parameters**:
    - `path`: The path to the CSV file.
    - `has_header`: Whether the string contains a header row.
    - `check_header`: Whether the function should try to determine whether the file has a header.
        If the file has a header row but you didn't set the `has_header` flag, an
        exception will be raised. If you're sure that the way you use the function
        is correct, you can disable this check using this parameter.
    - `encoding`: The encoding of the CSV file. If you wrote the CSV file using a library like pandas,
        you need to check the documentation to see what encoding they use by default
        when writing files (likely `"utf-8"` in which case this can be left at its default value).
    - `delimiter`: What delimiter is used in the the CSV file. Default is the comma.
    - `**kwargs`: Additional keyword arguments passed to the Python CSV parser. Refer to the
        [official documentation](https://docs.python.org/3/library/csv.html#csv-fmt-params)
        for a list of supported arguments.

    **Returns**:
    A BytesIO object that can be passed to the methods resposible for uploading data.
    """
    lines = []
    with open(path, 'r', encoding=encoding) as csvfile:
        return _read_input_csv(
            csvfile,
            has_header=has_header,
            check_header=check_header,
            **kwargs,
        )


def read_input_csv_string(
        content: str,
        /, *,
        has_header: bool = True,
        check_header: bool = True,
        **kwargs
) -> io.BytesIO:
    """
    Read CSV from a string and turn it into a bytes array of the correct format so that it can be uploaded
    to the Decentriq platform.

    **Parameters**:
    - `content`: The string containing the CSV file.
    - `has_header`: Whether the string contains a header row.
    - `check_header`: Whether the function should try to determine whether the file has a header.
        If the file has a header row but you didn't set the `has_header` flag, an
        exception will be raised. If you're sure that the way you use the function
        is correct, you can disable this check using this parameter.
    - `encoding`: The encoding of the source file. If you wrote the CSV file using a library like pandas,
        you need to check the documentation to see what encoding they use by default
        when writing files.
    - `delimiter`: What delimiter is used in the the CSV file. Default is the comma.
    - `**kwargs`: Additional keyword arguments passed to the Python CSV parser. What flags can be passed can be
        seen [here](https://docs.python.org/3/library/csv.html#csv-fmt-params).

    **Returns**:
    A BytesIO object that can be passed to the methods resposible for uploading data.
    """
    return _read_input_csv(
        io.StringIO(content.strip()),
        has_header=has_header,
        check_header=check_header,
        **kwargs
    )


def _read_input_csv(
        data: io.TextIOBase,
        /, *,
        has_header: bool = True,
        check_header: bool = True,
        **kwargs
) -> io.BytesIO:
    lines = []
    if check_header:
        sample = '\n'.join(data.readline() for _ in range(20))
        file_has_header = csv.Sniffer().has_header(sample)
        if file_has_header and has_header is False:
            raise Exception(
                "Warning: the file seems to have a header but the header flag is not set to true!"
                " This flag needs to be set correctly in order for the data to be uploaded."
            )
        elif not file_has_header and has_header is True:
            raise Exception(
                "Warning: the file doesn't seem to have a header but the header flag is set!"
                " This flag needs to be set correctly in order for the data to be uploaded."
            )
        data.seek(0)
    reader = csv.reader(data, **kwargs)
    if has_header:
        next(reader)
    for row in reader:
        lines.append(','.join(row))
    return io.BytesIO('\n'.join(lines).encode())


def upload_and_publish_dataset(
        data: io.BytesIO,
        key: Key,
        data_room_id: str,
        *,
        data_node: ValidatedDataNode,
        session: Session,
        description: str,
        validate: bool = True,
        **kwargs
) -> str:
    """
    Convenience function for uploading data and validating the schema of the uploaded
    data.

    Validation of tabular data is a separate computation for which specific compute nodes
    need to be present in the compute graph defined by the data room definition.
    This function will take care of triggering the validation action for you.
    In case validation fails, an exception will be raised. Validation can be turned off
    using the `validate` parameter.

    **Parameters**:
    - `data`: The input data to be uploaded. Use one of the reader functions provided in this
        package to read CSV-like data.
    - `key`: A key for encrypting the data to-be-uploaded.
    - `data_room_id`: To which data room the dataset should be published. This is the id you
        get when publishing a data room.
    - `data_node`: The data node object added to the data room builder.
    - `session`: The session with which to communicate with the enclave.
    - `description`: An identifier to the dataset.
    - `validate`: Whether to perform the validation operation.

    **Returns**:
    The manifest hash (dataset id) in case the upload and validation succeeded.
    """
    manifest_hash = session.client.upload_dataset(
        data,
        key,
        description=description,
    )
    session.publish_dataset(
        data_room_id, manifest_hash,
        leaf_name=data_node.input_node_name,
        key=key
    )

    if validate:
        try:
            session.run_computation_and_get_results(
                data_room_id,
                data_node.validation_computation_name,
                **kwargs
            )
        except Exception as e:
            raise Exception("Validation of dataset failed! Reason: {}".format(e))

    return manifest_hash


def _read_sql_query_result(result: bytes) -> Tuple[str, TableSchema]:
    archive = zipfile.ZipFile(io.BytesIO(result), "r")

    if set(archive.namelist()) != {"dataset.csv", "types"}:
        raise Exception(
            "The given result cannot be read as it doesn't contain all the required files."
            " Expected files: 'dataset.csv' and 'types', only found: [{}]".format(
                ', '.join(archive.namelist())
            )
        )

    csv = archive.read("dataset.csv").decode()
    schema = TableSchema()
    parse_length_delimited(archive.read("types"), schema)

    return (csv, schema)


def read_sql_query_result_as_list(result: bytes) -> Tuple[List[List[str]], TableSchema]:
    """
    Read the given raw CSV output from an SQL query result and transform it into a list of
    lists, where each inner list corresponds to a row in the tabular output data.
    The header row won't be contained in the resulting output and all the inner values
    will be of type string.
    The actual column types as well as their names will be returned as part of the schema
    value.
    """
    csv_content, schema = _read_sql_query_result(result)
    reader = csv.reader(io.StringIO(csv_content))
    return list(reader), schema


def read_sql_query_result_as_string(
        result: bytes,
        include_header: bool = True
) -> str:
    """
    Read the given raw CSV output from an SQL query result and transform it into a string
    containing the full CSV file including the header row.
    The resulting string can be written directly to a file from where it can be read
    using your data analysis library of choice.

    **Parameters**:
    - `result`: The result from which to extract a CSV file as a string.
    - `include_header`: Whether to include an additional header row that contains
        column names (provided for example by using alias expressions
        in SELECT statements). If this setting is true, but no column
        names could be found, the names "V1", "V2" and so will be used.
    """
    content, schema = _read_sql_query_result(result)

    if include_header:
        opt_columns = [col.name for col in schema.namedColumns]
        # Use V1, V2, ... for unknown column names => same format as R uses.
        replacement_columns = [f"V{ix}" for ix in range(1, len(opt_columns) + 1)]
        columns = [ c1 if c1 else c2 for c1, c2 in zip(opt_columns, replacement_columns)]
        returned_content = ','.join(columns) + '\n' + content
    else:
        returned_content = content

    # Remove trailing newlines
    return returned_content.strip()



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import data_room_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor = ...

class DataRoomStatus(_DataRoomStatus, metaclass=_DataRoomStatusEnumTypeWrapper):
    pass
class _DataRoomStatus:
    V = typing.NewType('V', builtins.int)
class _DataRoomStatusEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_DataRoomStatus.V], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor = ...
    Active = DataRoomStatus.V(0)
    Stopped = DataRoomStatus.V(1)

Active = DataRoomStatus.V(0)
Stopped = DataRoomStatus.V(1)
global___DataRoomStatus = DataRoomStatus


class GcgRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    EXECUTECOMPUTEREQUEST_FIELD_NUMBER: builtins.int
    CREATEDATAROOMREQUEST_FIELD_NUMBER: builtins.int
    PUBLISHDATASETTODATAROOMREQUEST_FIELD_NUMBER: builtins.int
    RETRIEVEDATAROOMREQUEST_FIELD_NUMBER: builtins.int
    RETRIEVEAUDITLOGREQUEST_FIELD_NUMBER: builtins.int
    RETRIEVEDATAROOMSTATUSREQUEST_FIELD_NUMBER: builtins.int
    UPDATEDATAROOMSTATUSREQUEST_FIELD_NUMBER: builtins.int
    RETRIEVEPUBLISHEDDATASETSREQUEST_FIELD_NUMBER: builtins.int
    REMOVEPUBLISHEDDATASETREQUEST_FIELD_NUMBER: builtins.int
    JOBSTATUSREQUEST_FIELD_NUMBER: builtins.int
    GETRESULTSREQUEST_FIELD_NUMBER: builtins.int
    @property
    def executeComputeRequest(self) -> global___ExecuteComputeRequest: ...
    @property
    def createDataRoomRequest(self) -> global___CreateDataRoomRequest: ...
    @property
    def publishDatasetToDataRoomRequest(self) -> global___PublishDatasetToDataRoomRequest: ...
    @property
    def retrieveDataRoomRequest(self) -> global___RetrieveDataRoomRequest: ...
    @property
    def retrieveAuditLogRequest(self) -> global___RetrieveAuditLogRequest: ...
    @property
    def retrieveDataRoomStatusRequest(self) -> global___RetrieveDataRoomStatusRequest: ...
    @property
    def updateDataRoomStatusRequest(self) -> global___UpdateDataRoomStatusRequest: ...
    @property
    def retrievePublishedDatasetsRequest(self) -> global___RetrievePublishedDatasetsRequest: ...
    @property
    def removePublishedDatasetRequest(self) -> global___RemovePublishedDatasetRequest: ...
    @property
    def jobStatusRequest(self) -> global___JobStatusRequest: ...
    @property
    def getResultsRequest(self) -> global___GetResultsRequest: ...
    def __init__(self,
        *,
        executeComputeRequest : typing.Optional[global___ExecuteComputeRequest] = ...,
        createDataRoomRequest : typing.Optional[global___CreateDataRoomRequest] = ...,
        publishDatasetToDataRoomRequest : typing.Optional[global___PublishDatasetToDataRoomRequest] = ...,
        retrieveDataRoomRequest : typing.Optional[global___RetrieveDataRoomRequest] = ...,
        retrieveAuditLogRequest : typing.Optional[global___RetrieveAuditLogRequest] = ...,
        retrieveDataRoomStatusRequest : typing.Optional[global___RetrieveDataRoomStatusRequest] = ...,
        updateDataRoomStatusRequest : typing.Optional[global___UpdateDataRoomStatusRequest] = ...,
        retrievePublishedDatasetsRequest : typing.Optional[global___RetrievePublishedDatasetsRequest] = ...,
        removePublishedDatasetRequest : typing.Optional[global___RemovePublishedDatasetRequest] = ...,
        jobStatusRequest : typing.Optional[global___JobStatusRequest] = ...,
        getResultsRequest : typing.Optional[global___GetResultsRequest] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"createDataRoomRequest",b"createDataRoomRequest",u"executeComputeRequest",b"executeComputeRequest",u"gcg_request",b"gcg_request",u"getResultsRequest",b"getResultsRequest",u"jobStatusRequest",b"jobStatusRequest",u"publishDatasetToDataRoomRequest",b"publishDatasetToDataRoomRequest",u"removePublishedDatasetRequest",b"removePublishedDatasetRequest",u"retrieveAuditLogRequest",b"retrieveAuditLogRequest",u"retrieveDataRoomRequest",b"retrieveDataRoomRequest",u"retrieveDataRoomStatusRequest",b"retrieveDataRoomStatusRequest",u"retrievePublishedDatasetsRequest",b"retrievePublishedDatasetsRequest",u"updateDataRoomStatusRequest",b"updateDataRoomStatusRequest"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"createDataRoomRequest",b"createDataRoomRequest",u"executeComputeRequest",b"executeComputeRequest",u"gcg_request",b"gcg_request",u"getResultsRequest",b"getResultsRequest",u"jobStatusRequest",b"jobStatusRequest",u"publishDatasetToDataRoomRequest",b"publishDatasetToDataRoomRequest",u"removePublishedDatasetRequest",b"removePublishedDatasetRequest",u"retrieveAuditLogRequest",b"retrieveAuditLogRequest",u"retrieveDataRoomRequest",b"retrieveDataRoomRequest",u"retrieveDataRoomStatusRequest",b"retrieveDataRoomStatusRequest",u"retrievePublishedDatasetsRequest",b"retrievePublishedDatasetsRequest",u"updateDataRoomStatusRequest",b"updateDataRoomStatusRequest"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal[u"gcg_request",b"gcg_request"]) -> typing.Optional[typing_extensions.Literal["executeComputeRequest","createDataRoomRequest","publishDatasetToDataRoomRequest","retrieveDataRoomRequest","retrieveAuditLogRequest","retrieveDataRoomStatusRequest","updateDataRoomStatusRequest","retrievePublishedDatasetsRequest","removePublishedDatasetRequest","jobStatusRequest","getResultsRequest"]]: ...
global___GcgRequest = GcgRequest

class GcgResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    FAILURE_FIELD_NUMBER: builtins.int
    EXECUTECOMPUTERESPONSE_FIELD_NUMBER: builtins.int
    CREATEDATAROOMRESPONSE_FIELD_NUMBER: builtins.int
    PUBLISHDATASETTODATAROOMRESPONSE_FIELD_NUMBER: builtins.int
    RETRIEVEDATAROOMRESPONSE_FIELD_NUMBER: builtins.int
    RETRIEVEAUDITLOGRESPONSE_FIELD_NUMBER: builtins.int
    RETRIEVEDATAROOMSTATUSRESPONSE_FIELD_NUMBER: builtins.int
    UPDATEDATAROOMSTATUSRESPONSE_FIELD_NUMBER: builtins.int
    RETRIEVEPUBLISHEDDATASETSRESPONSE_FIELD_NUMBER: builtins.int
    REMOVEPUBLISHEDDATASETRESPONSE_FIELD_NUMBER: builtins.int
    JOBSTATUSRESPONSE_FIELD_NUMBER: builtins.int
    GETRESULTSRESPONSECHUNK_FIELD_NUMBER: builtins.int
    GETRESULTSRESPONSEFOOTER_FIELD_NUMBER: builtins.int
    failure: typing.Text = ...
    @property
    def executeComputeResponse(self) -> global___ExecuteComputeResponse: ...
    @property
    def createDataRoomResponse(self) -> global___CreateDataRoomResponse: ...
    @property
    def publishDatasetToDataRoomResponse(self) -> global___PublishDatasetToDataRoomResponse: ...
    @property
    def retrieveDataRoomResponse(self) -> global___RetrieveDataRoomResponse: ...
    @property
    def retrieveAuditLogResponse(self) -> global___RetrieveAuditLogResponse: ...
    @property
    def retrieveDataRoomStatusResponse(self) -> global___RetrieveDataRoomStatusResponse: ...
    @property
    def updateDataRoomStatusResponse(self) -> global___UpdateDataRoomStatusResponse: ...
    @property
    def retrievePublishedDatasetsResponse(self) -> global___RetrievePublishedDatasetsResponse: ...
    @property
    def removePublishedDatasetResponse(self) -> global___RemovePublishedDatasetResponse: ...
    @property
    def jobStatusResponse(self) -> global___JobStatusResponse: ...
    @property
    def getResultsResponseChunk(self) -> global___GetResultsResponseChunk: ...
    @property
    def getResultsResponseFooter(self) -> global___GetResultsResponseFooter: ...
    def __init__(self,
        *,
        failure : typing.Text = ...,
        executeComputeResponse : typing.Optional[global___ExecuteComputeResponse] = ...,
        createDataRoomResponse : typing.Optional[global___CreateDataRoomResponse] = ...,
        publishDatasetToDataRoomResponse : typing.Optional[global___PublishDatasetToDataRoomResponse] = ...,
        retrieveDataRoomResponse : typing.Optional[global___RetrieveDataRoomResponse] = ...,
        retrieveAuditLogResponse : typing.Optional[global___RetrieveAuditLogResponse] = ...,
        retrieveDataRoomStatusResponse : typing.Optional[global___RetrieveDataRoomStatusResponse] = ...,
        updateDataRoomStatusResponse : typing.Optional[global___UpdateDataRoomStatusResponse] = ...,
        retrievePublishedDatasetsResponse : typing.Optional[global___RetrievePublishedDatasetsResponse] = ...,
        removePublishedDatasetResponse : typing.Optional[global___RemovePublishedDatasetResponse] = ...,
        jobStatusResponse : typing.Optional[global___JobStatusResponse] = ...,
        getResultsResponseChunk : typing.Optional[global___GetResultsResponseChunk] = ...,
        getResultsResponseFooter : typing.Optional[global___GetResultsResponseFooter] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"createDataRoomResponse",b"createDataRoomResponse",u"executeComputeResponse",b"executeComputeResponse",u"failure",b"failure",u"gcg_response",b"gcg_response",u"getResultsResponseChunk",b"getResultsResponseChunk",u"getResultsResponseFooter",b"getResultsResponseFooter",u"jobStatusResponse",b"jobStatusResponse",u"publishDatasetToDataRoomResponse",b"publishDatasetToDataRoomResponse",u"removePublishedDatasetResponse",b"removePublishedDatasetResponse",u"retrieveAuditLogResponse",b"retrieveAuditLogResponse",u"retrieveDataRoomResponse",b"retrieveDataRoomResponse",u"retrieveDataRoomStatusResponse",b"retrieveDataRoomStatusResponse",u"retrievePublishedDatasetsResponse",b"retrievePublishedDatasetsResponse",u"updateDataRoomStatusResponse",b"updateDataRoomStatusResponse"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"createDataRoomResponse",b"createDataRoomResponse",u"executeComputeResponse",b"executeComputeResponse",u"failure",b"failure",u"gcg_response",b"gcg_response",u"getResultsResponseChunk",b"getResultsResponseChunk",u"getResultsResponseFooter",b"getResultsResponseFooter",u"jobStatusResponse",b"jobStatusResponse",u"publishDatasetToDataRoomResponse",b"publishDatasetToDataRoomResponse",u"removePublishedDatasetResponse",b"removePublishedDatasetResponse",u"retrieveAuditLogResponse",b"retrieveAuditLogResponse",u"retrieveDataRoomResponse",b"retrieveDataRoomResponse",u"retrieveDataRoomStatusResponse",b"retrieveDataRoomStatusResponse",u"retrievePublishedDatasetsResponse",b"retrievePublishedDatasetsResponse",u"updateDataRoomStatusResponse",b"updateDataRoomStatusResponse"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal[u"gcg_response",b"gcg_response"]) -> typing.Optional[typing_extensions.Literal["failure","executeComputeResponse","createDataRoomResponse","publishDatasetToDataRoomResponse","retrieveDataRoomResponse","retrieveAuditLogResponse","retrieveDataRoomStatusResponse","updateDataRoomStatusResponse","retrievePublishedDatasetsResponse","removePublishedDatasetResponse","jobStatusResponse","getResultsResponseChunk","getResultsResponseFooter"]]: ...
global___GcgResponse = GcgResponse

class CreateDataRoomRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    DATAROOM_FIELD_NUMBER: builtins.int
    SCOPE_FIELD_NUMBER: builtins.int
    @property
    def dataRoom(self) -> data_room_pb2.DataRoom: ...
    scope: builtins.bytes = ...
    def __init__(self,
        *,
        dataRoom : typing.Optional[data_room_pb2.DataRoom] = ...,
        scope : builtins.bytes = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"dataRoom",b"dataRoom"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"dataRoom",b"dataRoom",u"scope",b"scope"]) -> None: ...
global___CreateDataRoomRequest = CreateDataRoomRequest

class CreateDataRoomResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    DATAROOMID_FIELD_NUMBER: builtins.int
    DATAROOMVALIDATIONERROR_FIELD_NUMBER: builtins.int
    dataRoomId: builtins.bytes = ...
    @property
    def dataRoomValidationError(self) -> global___DataRoomValidationError: ...
    def __init__(self,
        *,
        dataRoomId : builtins.bytes = ...,
        dataRoomValidationError : typing.Optional[global___DataRoomValidationError] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"create_data_room_response",b"create_data_room_response",u"dataRoomId",b"dataRoomId",u"dataRoomValidationError",b"dataRoomValidationError"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"create_data_room_response",b"create_data_room_response",u"dataRoomId",b"dataRoomId",u"dataRoomValidationError",b"dataRoomValidationError"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal[u"create_data_room_response",b"create_data_room_response"]) -> typing.Optional[typing_extensions.Literal["dataRoomId","dataRoomValidationError"]]: ...
global___CreateDataRoomResponse = CreateDataRoomResponse

class DataRoomValidationError(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    MESSAGE_FIELD_NUMBER: builtins.int
    COMPUTENODEINDEX_FIELD_NUMBER: builtins.int
    COLUMNINDEX_FIELD_NUMBER: builtins.int
    TABLEINDEX_FIELD_NUMBER: builtins.int
    USERPERMISSIONINDEX_FIELD_NUMBER: builtins.int
    PERMISSIONINDEX_FIELD_NUMBER: builtins.int
    ATTESTATIONSPECIFICATIONINDEX_FIELD_NUMBER: builtins.int
    AUTHENTICATIONMETHODINDEX_FIELD_NUMBER: builtins.int
    message: typing.Text = ...
    computeNodeIndex: builtins.int = ...
    columnIndex: builtins.int = ...
    tableIndex: builtins.int = ...
    userPermissionIndex: builtins.int = ...
    permissionIndex: builtins.int = ...
    attestationSpecificationIndex: builtins.int = ...
    authenticationMethodIndex: builtins.int = ...
    def __init__(self,
        *,
        message : typing.Text = ...,
        computeNodeIndex : builtins.int = ...,
        columnIndex : builtins.int = ...,
        tableIndex : builtins.int = ...,
        userPermissionIndex : builtins.int = ...,
        permissionIndex : builtins.int = ...,
        attestationSpecificationIndex : builtins.int = ...,
        authenticationMethodIndex : builtins.int = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"_attestationSpecificationIndex",b"_attestationSpecificationIndex",u"_authenticationMethodIndex",b"_authenticationMethodIndex",u"_columnIndex",b"_columnIndex",u"_computeNodeIndex",b"_computeNodeIndex",u"_permissionIndex",b"_permissionIndex",u"_tableIndex",b"_tableIndex",u"_userPermissionIndex",b"_userPermissionIndex",u"attestationSpecificationIndex",b"attestationSpecificationIndex",u"authenticationMethodIndex",b"authenticationMethodIndex",u"columnIndex",b"columnIndex",u"computeNodeIndex",b"computeNodeIndex",u"permissionIndex",b"permissionIndex",u"tableIndex",b"tableIndex",u"userPermissionIndex",b"userPermissionIndex"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"_attestationSpecificationIndex",b"_attestationSpecificationIndex",u"_authenticationMethodIndex",b"_authenticationMethodIndex",u"_columnIndex",b"_columnIndex",u"_computeNodeIndex",b"_computeNodeIndex",u"_permissionIndex",b"_permissionIndex",u"_tableIndex",b"_tableIndex",u"_userPermissionIndex",b"_userPermissionIndex",u"attestationSpecificationIndex",b"attestationSpecificationIndex",u"authenticationMethodIndex",b"authenticationMethodIndex",u"columnIndex",b"columnIndex",u"computeNodeIndex",b"computeNodeIndex",u"message",b"message",u"permissionIndex",b"permissionIndex",u"tableIndex",b"tableIndex",u"userPermissionIndex",b"userPermissionIndex"]) -> None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal[u"_attestationSpecificationIndex",b"_attestationSpecificationIndex"]) -> typing.Optional[typing_extensions.Literal["attestationSpecificationIndex"]]: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal[u"_authenticationMethodIndex",b"_authenticationMethodIndex"]) -> typing.Optional[typing_extensions.Literal["authenticationMethodIndex"]]: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal[u"_columnIndex",b"_columnIndex"]) -> typing.Optional[typing_extensions.Literal["columnIndex"]]: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal[u"_computeNodeIndex",b"_computeNodeIndex"]) -> typing.Optional[typing_extensions.Literal["computeNodeIndex"]]: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal[u"_permissionIndex",b"_permissionIndex"]) -> typing.Optional[typing_extensions.Literal["permissionIndex"]]: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal[u"_tableIndex",b"_tableIndex"]) -> typing.Optional[typing_extensions.Literal["tableIndex"]]: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal[u"_userPermissionIndex",b"_userPermissionIndex"]) -> typing.Optional[typing_extensions.Literal["userPermissionIndex"]]: ...
global___DataRoomValidationError = DataRoomValidationError

class PublishDatasetToDataRoomRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    DATASETHASH_FIELD_NUMBER: builtins.int
    DATAROOMID_FIELD_NUMBER: builtins.int
    LEAFNAME_FIELD_NUMBER: builtins.int
    ENCRYPTIONKEY_FIELD_NUMBER: builtins.int
    SCOPE_FIELD_NUMBER: builtins.int
    datasetHash: builtins.bytes = ...
    dataRoomId: builtins.bytes = ...
    leafName: typing.Text = ...
    encryptionKey: builtins.bytes = ...
    scope: builtins.bytes = ...
    def __init__(self,
        *,
        datasetHash : builtins.bytes = ...,
        dataRoomId : builtins.bytes = ...,
        leafName : typing.Text = ...,
        encryptionKey : builtins.bytes = ...,
        scope : builtins.bytes = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"dataRoomId",b"dataRoomId",u"datasetHash",b"datasetHash",u"encryptionKey",b"encryptionKey",u"leafName",b"leafName",u"scope",b"scope"]) -> None: ...
global___PublishDatasetToDataRoomRequest = PublishDatasetToDataRoomRequest

class PublishDatasetToDataRoomResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    def __init__(self,
        ) -> None: ...
global___PublishDatasetToDataRoomResponse = PublishDatasetToDataRoomResponse

class ExecuteComputeRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    DATAROOMID_FIELD_NUMBER: builtins.int
    COMPUTENODENAMES_FIELD_NUMBER: builtins.int
    ISDRYRUN_FIELD_NUMBER: builtins.int
    SCOPE_FIELD_NUMBER: builtins.int
    dataRoomId: builtins.bytes = ...
    @property
    def computeNodeNames(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[typing.Text]: ...
    isDryRun: builtins.bool = ...
    scope: builtins.bytes = ...
    def __init__(self,
        *,
        dataRoomId : builtins.bytes = ...,
        computeNodeNames : typing.Optional[typing.Iterable[typing.Text]] = ...,
        isDryRun : builtins.bool = ...,
        scope : builtins.bytes = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"computeNodeNames",b"computeNodeNames",u"dataRoomId",b"dataRoomId",u"isDryRun",b"isDryRun",u"scope",b"scope"]) -> None: ...
global___ExecuteComputeRequest = ExecuteComputeRequest

class ExecuteComputeResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    JOBID_FIELD_NUMBER: builtins.int
    jobId: builtins.bytes = ...
    def __init__(self,
        *,
        jobId : builtins.bytes = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"jobId",b"jobId"]) -> None: ...
global___ExecuteComputeResponse = ExecuteComputeResponse

class JobStatusRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    JOBID_FIELD_NUMBER: builtins.int
    jobId: builtins.bytes = ...
    def __init__(self,
        *,
        jobId : builtins.bytes = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"jobId",b"jobId"]) -> None: ...
global___JobStatusRequest = JobStatusRequest

class JobStatusResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    COMPLETECOMPUTENODENAMES_FIELD_NUMBER: builtins.int
    @property
    def completeComputeNodeNames(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[typing.Text]: ...
    def __init__(self,
        *,
        completeComputeNodeNames : typing.Optional[typing.Iterable[typing.Text]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"completeComputeNodeNames",b"completeComputeNodeNames"]) -> None: ...
global___JobStatusResponse = JobStatusResponse

class GetResultsRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    JOBID_FIELD_NUMBER: builtins.int
    COMPUTENODENAME_FIELD_NUMBER: builtins.int
    jobId: builtins.bytes = ...
    computeNodeName: typing.Text = ...
    def __init__(self,
        *,
        jobId : builtins.bytes = ...,
        computeNodeName : typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"computeNodeName",b"computeNodeName",u"jobId",b"jobId"]) -> None: ...
global___GetResultsRequest = GetResultsRequest

class GetResultsResponseChunk(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    DATA_FIELD_NUMBER: builtins.int
    data: builtins.bytes = ...
    def __init__(self,
        *,
        data : builtins.bytes = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"data",b"data"]) -> None: ...
global___GetResultsResponseChunk = GetResultsResponseChunk

class GetResultsResponseFooter(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    def __init__(self,
        ) -> None: ...
global___GetResultsResponseFooter = GetResultsResponseFooter

class RetrieveDataRoomRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    DATAROOMID_FIELD_NUMBER: builtins.int
    SCOPE_FIELD_NUMBER: builtins.int
    dataRoomId: builtins.bytes = ...
    scope: builtins.bytes = ...
    def __init__(self,
        *,
        dataRoomId : builtins.bytes = ...,
        scope : builtins.bytes = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"dataRoomId",b"dataRoomId",u"scope",b"scope"]) -> None: ...
global___RetrieveDataRoomRequest = RetrieveDataRoomRequest

class RetrieveDataRoomResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    DATAROOM_FIELD_NUMBER: builtins.int
    @property
    def dataRoom(self) -> data_room_pb2.DataRoom: ...
    def __init__(self,
        *,
        dataRoom : typing.Optional[data_room_pb2.DataRoom] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"dataRoom",b"dataRoom"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"dataRoom",b"dataRoom"]) -> None: ...
global___RetrieveDataRoomResponse = RetrieveDataRoomResponse

class RetrieveAuditLogRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    DATAROOMID_FIELD_NUMBER: builtins.int
    SCOPE_FIELD_NUMBER: builtins.int
    dataRoomId: builtins.bytes = ...
    scope: builtins.bytes = ...
    def __init__(self,
        *,
        dataRoomId : builtins.bytes = ...,
        scope : builtins.bytes = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"dataRoomId",b"dataRoomId",u"scope",b"scope"]) -> None: ...
global___RetrieveAuditLogRequest = RetrieveAuditLogRequest

class RetrieveAuditLogResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    LOG_FIELD_NUMBER: builtins.int
    log: builtins.bytes = ...
    def __init__(self,
        *,
        log : builtins.bytes = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"log",b"log"]) -> None: ...
global___RetrieveAuditLogResponse = RetrieveAuditLogResponse

class RetrieveDataRoomStatusRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    DATAROOMID_FIELD_NUMBER: builtins.int
    SCOPE_FIELD_NUMBER: builtins.int
    dataRoomId: builtins.bytes = ...
    scope: builtins.bytes = ...
    def __init__(self,
        *,
        dataRoomId : builtins.bytes = ...,
        scope : builtins.bytes = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"dataRoomId",b"dataRoomId",u"scope",b"scope"]) -> None: ...
global___RetrieveDataRoomStatusRequest = RetrieveDataRoomStatusRequest

class RetrieveDataRoomStatusResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    STATUS_FIELD_NUMBER: builtins.int
    status: global___DataRoomStatus.V = ...
    def __init__(self,
        *,
        status : global___DataRoomStatus.V = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"status",b"status"]) -> None: ...
global___RetrieveDataRoomStatusResponse = RetrieveDataRoomStatusResponse

class UpdateDataRoomStatusRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    DATAROOMID_FIELD_NUMBER: builtins.int
    SCOPE_FIELD_NUMBER: builtins.int
    STATUS_FIELD_NUMBER: builtins.int
    dataRoomId: builtins.bytes = ...
    scope: builtins.bytes = ...
    status: global___DataRoomStatus.V = ...
    def __init__(self,
        *,
        dataRoomId : builtins.bytes = ...,
        scope : builtins.bytes = ...,
        status : global___DataRoomStatus.V = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"dataRoomId",b"dataRoomId",u"scope",b"scope",u"status",b"status"]) -> None: ...
global___UpdateDataRoomStatusRequest = UpdateDataRoomStatusRequest

class UpdateDataRoomStatusResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    def __init__(self,
        ) -> None: ...
global___UpdateDataRoomStatusResponse = UpdateDataRoomStatusResponse

class RetrievePublishedDatasetsRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    DATAROOMID_FIELD_NUMBER: builtins.int
    SCOPE_FIELD_NUMBER: builtins.int
    dataRoomId: builtins.bytes = ...
    scope: builtins.bytes = ...
    def __init__(self,
        *,
        dataRoomId : builtins.bytes = ...,
        scope : builtins.bytes = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"dataRoomId",b"dataRoomId",u"scope",b"scope"]) -> None: ...
global___RetrievePublishedDatasetsRequest = RetrievePublishedDatasetsRequest

class PublishedDataset(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    LEAFNAME_FIELD_NUMBER: builtins.int
    USER_FIELD_NUMBER: builtins.int
    TIMESTAMP_FIELD_NUMBER: builtins.int
    DATASETHASH_FIELD_NUMBER: builtins.int
    leafName: typing.Text = ...
    user: typing.Text = ...
    timestamp: builtins.int = ...
    datasetHash: builtins.bytes = ...
    def __init__(self,
        *,
        leafName : typing.Text = ...,
        user : typing.Text = ...,
        timestamp : builtins.int = ...,
        datasetHash : builtins.bytes = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"datasetHash",b"datasetHash",u"leafName",b"leafName",u"timestamp",b"timestamp",u"user",b"user"]) -> None: ...
global___PublishedDataset = PublishedDataset

class RetrievePublishedDatasetsResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    PUBLISHEDDATASETS_FIELD_NUMBER: builtins.int
    @property
    def publishedDatasets(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___PublishedDataset]: ...
    def __init__(self,
        *,
        publishedDatasets : typing.Optional[typing.Iterable[global___PublishedDataset]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"publishedDatasets",b"publishedDatasets"]) -> None: ...
global___RetrievePublishedDatasetsResponse = RetrievePublishedDatasetsResponse

class RemovePublishedDatasetRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    DATAROOMID_FIELD_NUMBER: builtins.int
    LEAFNAME_FIELD_NUMBER: builtins.int
    SCOPE_FIELD_NUMBER: builtins.int
    dataRoomId: builtins.bytes = ...
    leafName: typing.Text = ...
    scope: builtins.bytes = ...
    def __init__(self,
        *,
        dataRoomId : builtins.bytes = ...,
        leafName : typing.Text = ...,
        scope : builtins.bytes = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"dataRoomId",b"dataRoomId",u"leafName",b"leafName",u"scope",b"scope"]) -> None: ...
global___RemovePublishedDatasetRequest = RemovePublishedDatasetRequest

class RemovePublishedDatasetResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    def __init__(self,
        ) -> None: ...
global___RemovePublishedDatasetResponse = RemovePublishedDatasetResponse

class DriverTaskConfig(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    NOOP_FIELD_NUMBER: builtins.int
    STATICCONTENT_FIELD_NUMBER: builtins.int
    @property
    def noop(self) -> global___NoopConfig: ...
    @property
    def staticContent(self) -> global___StaticContentConfig: ...
    def __init__(self,
        *,
        noop : typing.Optional[global___NoopConfig] = ...,
        staticContent : typing.Optional[global___StaticContentConfig] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"driver_task_config",b"driver_task_config",u"noop",b"noop",u"staticContent",b"staticContent"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"driver_task_config",b"driver_task_config",u"noop",b"noop",u"staticContent",b"staticContent"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal[u"driver_task_config",b"driver_task_config"]) -> typing.Optional[typing_extensions.Literal["noop","staticContent"]]: ...
global___DriverTaskConfig = DriverTaskConfig

class NoopConfig(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    def __init__(self,
        ) -> None: ...
global___NoopConfig = NoopConfig

class StaticContentConfig(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    CONTENT_FIELD_NUMBER: builtins.int
    content: builtins.bytes = ...
    def __init__(self,
        *,
        content : builtins.bytes = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"content",b"content"]) -> None: ...
global___StaticContentConfig = StaticContentConfig



from __future__ import annotations
import chily
import hmac
import json
from base64 import b64encode, b64decode
from cryptography.hazmat.primitives.kdf.hkdf import HKDF
from cryptography.hazmat.primitives import hashes
from typing import Any, List, Tuple, TYPE_CHECKING, Iterator, Optional
from time import sleep
from .api import Endpoints
from .authentication import Auth, Sigma
from .proto import DataRoom
from .proto import (DataNoncePubkey, Request, Response)
from .proto import (
    CreateDataRoomRequest, CreateDataRoomResponse,
    ExecuteComputeRequest, ExecuteComputeResponse, GcgRequest, GcgResponse, GetResultsRequest,
    GetResultsResponseChunk, JobStatusRequest, JobStatusResponse,
    PublishDatasetToDataRoomRequest, PublishDatasetToDataRoomResponse,
    RemovePublishedDatasetRequest, RemovePublishedDatasetResponse,
    RetrieveAuditLogRequest, RetrieveAuditLogResponse, RetrieveDataRoomRequest,
    RetrieveDataRoomResponse, RetrieveDataRoomStatusRequest,
    RetrieveDataRoomStatusResponse, RetrievePublishedDatasetsRequest,
    RetrievePublishedDatasetsResponse, UpdateDataRoomStatusRequest,
    UpdateDataRoomStatusResponse, DataRoomStatus, AttestationSpecification, Fatquote
)
from .proto.length_delimited import parse_length_delimited, serialize_length_delimited
from .storage import Key
from .types import FatquoteResBody, EnclaveMessage, ScopeTypes, JobId
from .verification import QuoteBody, Verification
from .helpers import get_data_room_id


if TYPE_CHECKING:
    from .client import Client


def datanoncepubkey_to_message(
        encrypted_data: bytes,
        nonce: bytes,
        pubkey: bytes,
        sigma_auth: Sigma
) -> bytes:
    message = DataNoncePubkey()
    message.data = encrypted_data
    message.nonce = nonce
    message.pubkey = pubkey
    message.pki.certChainPem = sigma_auth.get_cert_chain()
    message.pki.signature = sigma_auth.get_signature()
    message.pki.idMac = sigma_auth.get_mac_tag()
    return serialize_length_delimited(message)


def message_to_datanoncepubkey(message: bytes) -> Tuple[bytes, bytes, bytes]:
    parsed_msg = DataNoncePubkey()
    parse_length_delimited(message, parsed_msg)
    return (parsed_msg.data, parsed_msg.nonce, parsed_msg.pubkey)


class Session():
    """
    Class for managing the communication with an enclave.
    """
    client: Client
    session_id: str
    enclave_identifier: str
    auth: Auth
    email: str
    keypair: Any
    fatquote: Fatquote
    quote: QuoteBody
    driver_attestation_specification: AttestationSpecification

    def __init__(
            self,
            client: Client,
            session_id: str,
            driver_attestation_specification: AttestationSpecification,
            auth: Auth,
            email: str
    ):
        """
        `Session` instances should not be instantiated directly but rather
         be created using a `Client` object using  `decentriq_platform.Client.create_session`.
        """
        url = Endpoints.SESSION_FATQUOTE.replace(":sessionId", session_id)
        response: FatquoteResBody = client._api.get(url).json()
        fatquote_bytes = b64decode(response["fatquoteBase64"])
        fatquote = Fatquote()
        fatquote.ParseFromString(fatquote_bytes)
        verification = Verification(attestation_specification=driver_attestation_specification)
        report_data = verification.verify(fatquote)
        self.client = client
        self.session_id = session_id
        self.auth = auth
        self.email = email
        self.keypair = chily.Keypair.from_random()
        self.fatquote = fatquote
        self.report_data = report_data
        self.driver_attestation_specification = driver_attestation_specification

    def _get_enclave_pubkey(self):
        pub_keyB = bytearray(self.report_data[:32])
        return chily.PublicKey.from_bytes(pub_keyB)

    def _encrypt_and_encode_data(self, data: bytes, auth: Auth) -> bytes:
        nonce = chily.Nonce.from_random()
        cipher = chily.Cipher(
            self.keypair.secret, self._get_enclave_pubkey()
        )
        enc_data = cipher.encrypt(data, nonce)
        public_keys = bytes(self.keypair.public_key.bytes) + bytes(self._get_enclave_pubkey().bytes)
        signature = auth._sign(public_keys)
        shared_key = bytes(self.keypair.secret.diffie_hellman(self._get_enclave_pubkey()).bytes)
        hkdf = HKDF(algorithm=hashes.SHA512(), length=64, info=b"IdP KDF Context", salt=b"")
        mac_key = hkdf.derive(shared_key)
        mac_tag = hmac.digest(mac_key, auth._get_user_id().encode(), "sha512")
        sigma_auth = Sigma(signature, mac_tag, auth)
        return datanoncepubkey_to_message(
            bytes(enc_data),
            bytes(nonce.bytes),
            bytes(self.keypair.public_key.bytes),
            sigma_auth
        )

    def _decode_and_decrypt_data(self, data: bytes) -> bytes:
        dec_data, nonceB, _ = message_to_datanoncepubkey(data)
        cipher = chily.Cipher(
            self.keypair.secret, self._get_enclave_pubkey()
        )
        return cipher.decrypt(dec_data, chily.Nonce.from_bytes(nonceB))

    def send_request(
            self,
            request: GcgRequest,
    ) -> List[GcgResponse]:
        """
        Low-level method for sending a raw `GcgRequest` to the enclave.
        Use this method if any of the convenience methods (such as `run_computation`) don't perform
        the exact task you want.
        """
        serialized_request = serialize_length_delimited(
            Request(
                deltaRequest=self._encrypt_and_encode_data(
                    serialize_length_delimited(request),
                    self.auth
                )
            )
        )
        url = Endpoints.SESSION_MESSAGES.replace(":sessionId", self.session_id)
        enclave_request = EnclaveMessage(data=b64encode(serialized_request).decode("ascii"))
        enclave_response: EnclaveMessage = self.client._api.post(
            url, json.dumps(enclave_request), {"Content-type": "application/json"}
        ).json()
        enclave_response_bytes = b64decode(enclave_response["data"])

        responses: List[GcgResponse] = []
        offset = 0
        while offset < len(enclave_response_bytes):
            response_container = Response()
            offset += parse_length_delimited(enclave_response_bytes[offset:], response_container)
            if response_container.HasField("unsuccessfulResponse"):
                raise Exception(response_container.unsuccessfulResponse)
            else:
                response = GcgResponse()
                decrypted_response = self._decode_and_decrypt_data(
                    response_container.successfulResponse
                )
                parse_length_delimited(decrypted_response, response)
                if response.HasField("failure"):
                    raise Exception(response.failure)
                responses.append(response)
        return responses

    def _publish_data_room(
            self,
            data_room_definition: DataRoom
    ) -> CreateDataRoomResponse:
        """
        Create a DataRoom with the provided protobuf `data_room` configuration object
        """
        scope_id = self.client._ensure_scope_with_metadata(self.email, {"type": ScopeTypes.DATA_ROOM_DEFINITION})
        request = CreateDataRoomRequest(
            dataRoom=data_room_definition,
            scope=bytes.fromhex(scope_id)
        )
        responses = self.send_request(GcgRequest(createDataRoomRequest=request))
        if len(responses) != 1:
            raise Exception("Malformed response")

        response = responses[0]

        if response.HasField("createDataRoomResponse"):
            if response.createDataRoomResponse.HasField("dataRoomValidationError"):
                raise Exception(
                    "Error when validating data room: {} (compute node index {})".format(
                        response.createDataRoomResponse.dataRoomValidationError.message,
                        response.createDataRoomResponse.dataRoomValidationError.computeNodeIndex
                    )
                )
        else:
            raise Exception(
                "Expected createDataRoomResponse, got "
                + str(response.WhichOneof("gcg_response"))
            )

        return response.createDataRoomResponse

    def publish_data_room(self, data_room_definition: DataRoom) -> str:
        """
        Publish the given data room making it immutable.

        The id returned from this method will be used when interacting with the
        published data room (for example when running computations or publishing datasets).
        """
        response = self._publish_data_room(data_room_definition)
        return get_data_room_id(response).hex()

    def stop_data_room(self, data_room_id: str):
        """
        Stop the data room with the given id, making it impossible to run new computations.
        """
        self._update_data_room_status(data_room_id, DataRoomStatus.Value("Stopped"))

    def _update_data_room_status(
            self,
            data_room_id: str,
            status # type: DataRoomStatus.V
    ) -> UpdateDataRoomStatusResponse:
        """
        Update the status of the data room.

        For the special case of stopping a data room, the method `stop_data_room` can be used.
        """
        data_room_id_bytes = bytes.fromhex(data_room_id)
        scope_id = self.client._ensure_scope_with_metadata(
            self.email,
            {
                "type": ScopeTypes.DATA_ROOM_INTERMEDIATE_DATA,
                "data_room_id": data_room_id
            }
        )
        request = UpdateDataRoomStatusRequest(
            dataRoomId=data_room_id_bytes,
            status=status,
            scope=bytes.fromhex(scope_id)
        )
        responses = self.send_request(GcgRequest(updateDataRoomStatusRequest=request))

        if len(responses) != 1:
            raise Exception("Malformed response")

        response = responses[0]
        if response.HasField("updateDataRoomStatusResponse"):
            pass
        else:
            raise Exception(
                "Expected updateDataRoomStatusResponse, got "
                + str(response.WhichOneof("gcg_response"))
            )

        return response.updateDataRoomStatusResponse

    def retrieve_data_room_status(
            self,
            data_room_id: str
    ) -> str:
        """
        Returns the status of the DataRoom. Valid values are `"Active"` or `"Stopped"`.
        """
        scope_id = self.client._ensure_scope_with_metadata(
            self.email,
            {
                "type": ScopeTypes.DATA_ROOM_INTERMEDIATE_DATA,
                "data_room_id": data_room_id,
            }
        )
        request = RetrieveDataRoomStatusRequest(
            dataRoomId=bytes.fromhex(data_room_id),
            scope=bytes.fromhex(scope_id)
        )
        responses = self.send_request(GcgRequest(retrieveDataRoomStatusRequest=request))
        if len(responses) != 1:
            raise Exception("Malformed response")
        response = responses[0]
        if not response.HasField("retrieveDataRoomStatusResponse"):
            raise Exception(
                "Expected retrieveDataRoomStatusResponse, got "
                + str(response.WhichOneof("gcg_response"))
            )

        return DataRoomStatus.Name(response.retrieveDataRoomStatusResponse.status)

    def retrieve_data_room_definition(
            self,
            data_room_id: str
    ) -> RetrieveDataRoomResponse:
        """
        Returns the underlying protobuf configuration object for the DataRoom
        """
        scope_id = self.client._ensure_scope_with_metadata(
            self.email,
            {
                "type": ScopeTypes.DATA_ROOM_INTERMEDIATE_DATA,
                "data_room_id": data_room_id
            }
        )
        request = RetrieveDataRoomRequest(
            dataRoomId=bytes.fromhex(data_room_id),
            scope=bytes.fromhex(scope_id)
        )
        responses = self.send_request(GcgRequest(retrieveDataRoomRequest=request))
        if len(responses) != 1:
            raise Exception("Malformed response")
        response = responses[0]
        if not response.HasField("retrieveDataRoomResponse"):
            raise Exception(
                "Expected retrieveDataRoomResponse, got "
                + str(response.WhichOneof("gcg_response"))
            )
        return response.retrieveDataRoomResponse

    def retrieve_audit_log(
            self,
            data_room_id: str
    ) -> RetrieveAuditLogResponse:
        """
        Returns the audit log for the DataRoom
        """
        scope_id = self.client._ensure_scope_with_metadata(
            self.email,
            {
                "type": ScopeTypes.DATA_ROOM_INTERMEDIATE_DATA,
                "data_room_id": data_room_id,
            }
        )
        request = RetrieveAuditLogRequest(
            dataRoomId=bytes.fromhex(data_room_id),
            scope=bytes.fromhex(scope_id)
        )
        responses = self.send_request(GcgRequest(retrieveAuditLogRequest=request))
        if len(responses) != 1:
            raise Exception("Malformed response")
        response = responses[0]
        if not response.HasField("retrieveAuditLogResponse"):
            raise Exception(
                "Expected retrieveAuditLogResponse, got "
                + str(response.WhichOneof("gcg_response"))
            )
        return response.retrieveAuditLogResponse

    def publish_dataset(
            self,
            data_room_id: str,
            manifest_hash: str,
            leaf_name: str,
            key: Key,
    ) -> PublishDatasetToDataRoomResponse:
        """
        Publishes a file to the DataRoom
        """
        scope_id = self.client._ensure_scope_with_metadata(
            self.email,
            {
                "type": ScopeTypes.DATA_ROOM_INTERMEDIATE_DATA,
                "data_room_id": data_room_id,
            }
        )
        request = PublishDatasetToDataRoomRequest(
            dataRoomId=bytes.fromhex(data_room_id),
            datasetHash=bytes.fromhex(manifest_hash),
            leafName=leaf_name,
            encryptionKey=key.material,
            scope=bytes.fromhex(scope_id)
        )
        responses = self.send_request(GcgRequest(publishDatasetToDataRoomRequest=request))
        if len(responses) != 1:
            raise Exception("Malformed response")
        response = responses[0]
        if not response.HasField("publishDatasetToDataRoomResponse"):
            raise Exception(
                "Expected publishDatasetToDataRoomResponse, got "
                + str(response.WhichOneof("gcg_response"))
            )
        return response.publishDatasetToDataRoomResponse

    def remove_published_dataset(
            self,
            data_room_id: str,
            leaf_name: str,
    ) -> RemovePublishedDatasetResponse:
        """
        Removes a published file from the DataRoom
        """
        scope_id = self.client._ensure_scope_with_metadata(
            self.email,
            {
                "type": ScopeTypes.DATA_ROOM_INTERMEDIATE_DATA,
                "data_room_id": data_room_id,
            }
        )
        request = RemovePublishedDatasetRequest(
            dataRoomId=bytes.fromhex(data_room_id),
            leafName=leaf_name,
            scope=bytes.fromhex(scope_id)
        )
        responses = self.send_request(GcgRequest(removePublishedDatasetRequest=request))
        if len(responses) != 1:
            raise Exception("Malformed response")
        response = responses[0]
        if not response.HasField("removePublishedDatasetResponse"):
            raise Exception(
                "Expected removePublishedDatasetResponse, got "
                + str(response.WhichOneof("gcg_response"))
            )
        return response.removePublishedDatasetResponse

    def retrieve_published_datasets(
            self,
            data_room_id: str,
    ) -> RetrievePublishedDatasetsResponse:
        """
        Returns the files published to the DataRoom
        """
        scope_id = self.client._ensure_scope_with_metadata(
            self.email,
            {
                "type": ScopeTypes.DATA_ROOM_INTERMEDIATE_DATA,
                "data_room_id": data_room_id,
            }
        )
        request = RetrievePublishedDatasetsRequest(
            dataRoomId=bytes.fromhex(data_room_id),
            scope=bytes.fromhex(scope_id)
        )
        responses = self.send_request(GcgRequest(retrievePublishedDatasetsRequest=request))
        if len(responses) != 1:
            raise Exception("Malformed response")
        response = responses[0]
        if not response.HasField("retrievePublishedDatasetsResponse"):
            raise Exception(
                "Expected retrievePublishedDatasetResponse, got "
                + str(response.WhichOneof("gcg_response"))
            )
        return response.retrievePublishedDatasetsResponse

    def _submit_compute(
            self,
            data_room_id: str,
            goal_nodes: List[str],
            /, *,
            dry_run: bool = False,
    ) -> ExecuteComputeResponse:
        """
        Submits a computation request which will generate an execution plan to
        perform the computation of the goal nodes
        """
        scope_id = self.client._ensure_scope_with_metadata(
            self.email,
            {
                "type": ScopeTypes.DATA_ROOM_INTERMEDIATE_DATA,
                "data_room_id": data_room_id,
            }
        )
        request = ExecuteComputeRequest(
            dataRoomId=bytes.fromhex(data_room_id),
            computeNodeNames=goal_nodes,
            isDryRun=dry_run,
            scope=bytes.fromhex(scope_id)
        )
        responses = self.send_request(GcgRequest(executeComputeRequest=request))
        if len(responses) != 1:
            raise Exception("Malformed response")
        response = responses[0]
        if not response.HasField("executeComputeResponse"):
            raise Exception(
                "Expected executeComputeResponse, got "
                + str(response.WhichOneof("gcg_response"))
            )
        return response.executeComputeResponse

    def get_computation_status(self, job_id: str) -> JobStatusResponse:
        """
        Returns the status of the provided `job_id` which will include the names
        of the nodes that completed their execution
        """
        request = JobStatusRequest(
            jobId=bytes.fromhex(job_id),
        )
        responses = self.send_request(GcgRequest(jobStatusRequest=request))
        if len(responses) != 1:
            raise Exception("Malformed response")
        response = responses[0]
        if not response.HasField("jobStatusResponse"):
            raise Exception(
                "Expected jobStatusResponse, got "
                + str(response.WhichOneof("gcg_response"))
            )
        return response.jobStatusResponse

    def _stream_job_results(
            self,
            job_id: bytes,
            compute_node_name: str,
    ) -> Iterator[GetResultsResponseChunk]:
        """
        Streams the results of the provided `job_id`
        """
        request = GetResultsRequest(
            jobId=job_id,
            computeNodeName=compute_node_name,
        )
        responses = self.send_request(GcgRequest(getResultsRequest=request))
        for response in responses:
            if response.HasField("getResultsResponseChunk"):
                yield response.getResultsResponseChunk
            elif response.HasField("getResultsResponseFooter"):
                return
            else:
                raise Exception(
                    "Expected getResultsResponseChunk or getResultsResponseFooter, got "
                    + str(response.WhichOneof("gcg_response"))
                )
        raise Exception("Enclave connection aborted while streaming results")

    def _get_job_results(
            self,
            job_id: bytes,
            compute_node_name: str,
    ) -> bytes:
        """
        Returns the results of the provided `job_id`
        """
        return b"".join(list(map(lambda chunk: chunk.data, self._stream_job_results(job_id, compute_node_name))))

    def run_computation(
            self,
            data_room_id: str,
            compute_node_name: str,
            /, *,
            dry_run: bool = False,
    ) -> JobId:
        """
        Run a specific computation within the data room with the given id.

        The result will be an identifier object of the job executing the computation.
        This object is required for checking a job's status and retrieving its results.
        """
        response = self._submit_compute(data_room_id, [compute_node_name], dry_run=dry_run)
        return JobId(response.jobId.hex(), compute_node_name)

    def get_computation_result(
            self,
            job_id: JobId,
            /, *,
            interval: int = 5,
            timeout: int = None
    ) -> bytes:
        """
        Wait for the given job to complete and retrieve its results as a raw byte string.

        The method will check for the job's completeness every `interval` seconds and up to
        an optional `timeout` seconds after which the method will raise an exception.
        If the job completes and the results can be retrieved successfully, a raw byte string
        will be returned. The bytes tring can be transformed into a more useful object using
        a variety of helper methods. These helper methods are specific for the type of computation
        you ran and can be found in the corresponding packages.
        """
        elapsed = 0
        job_id_bytes = bytes.fromhex(job_id.id)
        while True:
            if timeout is not None and elapsed > timeout:
                raise Exception(
                    f"Timeout when trying to get result for job {job_id.id} of {job_id.compute_node_name} (waited {timeout} seconds)"
                )
            elif job_id.compute_node_name in self.get_computation_status(job_id.id).completeComputeNodeNames:
                results = self._get_job_results(job_id_bytes, job_id.compute_node_name)
                return results
            else:
                sleep(interval)
                elapsed += interval

    def run_computation_and_get_results(
            self,
            data_room_id: str,
            compute_node_name: str,
            /, *,
            interval: int = 5,
            timeout: int = None
    ) -> Optional[bytes]:
        """
        Run a specific computation and return its results.

        This method is simply a wrapper for running `run_computation` and `get_computation_result`
        directly after each other
        """
        job_id = self.run_computation(data_room_id, compute_node_name)
        return self.get_computation_result(
            job_id,
            interval=interval,
            timeout=timeout
        )



from cryptography import x509
from cryptography.x509.oid import NameOID
from cryptography.hazmat.primitives import hashes, serialization
from cryptography.hazmat.primitives.asymmetric import padding, rsa

__all__ = ["Auth"]

PKey = rsa.RSAPrivateKey;

class Auth:
    """
    This class wraps the certificate used to identify a user and implements the
    signing of the messages that are sent to the enclave
    """
    def __init__(
            self,
            certificate_chain: bytes,
            keypair: PKey,
            user_id: str,
    ):
        """
        Create an authentication object with the supplied certificate chain and
        keypair. To use the identity provider of the decentriq platform use
        `decentriq_platform.Client.create_auth`
        """
        self.certificate_chain = certificate_chain
        self.kp = keypair
        self.user_id = user_id

    def _get_user_id(self) -> str:
        return self.user_id

    def _sign(self, data: bytes) -> bytes:
        return self.kp.sign(data, padding.PKCS1v15(), hashes.SHA512())

    def get_certificate_chain_pem(self) -> bytes:
        """
        Returns the chain of certificates in PEM format
        """
        return self.certificate_chain

class Sigma:
    def __init__(self, signature: bytes, mac_tag: bytes, auth_pki: Auth):
        self.signature: bytes = signature
        self.mac_tag: bytes = mac_tag
        self.auth_pki: Auth = auth_pki

    def get_mac_tag(self) -> bytes:
        return self.mac_tag

    def get_signature(self) -> bytes:
        return self.signature

    def get_cert_chain(self) -> bytes:
        return self.auth_pki.get_certificate_chain_pem()


def generate_key(bit_size: int = 4096) -> PKey:
    key = rsa.generate_private_key(
        public_exponent=65537,
        key_size=bit_size,
    )
    return key


def generate_csr(user_email: str, key: PKey) -> bytes:
    csr_builder = x509.CertificateSigningRequestBuilder()
    csr = csr_builder.subject_name(x509.Name([
        x509.NameAttribute(NameOID.COMMON_NAME, user_email)
    ])).add_extension(
        x509.BasicConstraints(ca=False, path_length=None), critical=True
    ).add_extension(
        x509.KeyUsage(
            digital_signature=True,
            content_commitment=True,
            key_encipherment=True,
            data_encipherment=False,
            key_agreement=False,
            key_cert_sign=False,
            crl_sign=False,
            encipher_only=False,
            decipher_only=False
        ),
        critical=True
    ).sign(key, hashes.SHA512())
    return csr.public_bytes(serialization.Encoding.PEM)



from ..proto.attestation_pb2 import (
    AttestationSpecification,
    AttestationSpecificationIntelEpid
)
from ..types import EnclaveSpecification
from ..attestation import intel_sgx_ias_root_ca_der


class EnclaveSpecifications:
    """
    The list of available `EnclaveSpecification`s provided by this compute package.

    Refer to the class with the same name located in the main package for more information.
    """
    V1_SGX = [
        EnclaveSpecification(
            name="decentriq.sql-worker",
            version="1",
            proto=AttestationSpecification(
                intelEpid=AttestationSpecificationIntelEpid(
                    mrenclave=bytes.fromhex("16506a96801ca841e784eeb55f6c162c2ef141667f1a1030df23ab61f69e0873"),
                    iasRootCaDer=intel_sgx_ias_root_ca_der
                )
            )
        )
    ]



from typing import List

from .proto.compute_container_pb2 import ContainerWorkerConfiguration, MountPoint
from ..proto.length_delimited import serialize_length_delimited
from ..proto import ComputeNodeFormat


class StaticContainerCompute():
    """
    Computation node which allows execution of programs inside a fixed
    container image
    """
    config: bytes
    """Serialized configuration to use in the compute node definition"""
    dependencies: List[str]
    """List of dependencies"""

    def __init__(
            self,
            command: List[str],
            mount_points: List[MountPoint],
            output_path: str,
            include_container_logs_on_error: bool
    ) -> None:
        configuration = ContainerWorkerConfiguration()
        configuration.static.command.extend(command)
        configuration.static.mountPoints.extend(mount_points)
        configuration.static.outputPath = output_path
        configuration.static.includeContainerLogsOnError = include_container_logs_on_error
        self.config = serialize_length_delimited(configuration)
        self.dependencies = list(map(lambda a: a.dependency, mount_points))
        self.enclave_type = "decentriq.python-worker"
        self.output_format = ComputeNodeFormat.ZIP



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: gcg.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import data_room_pb2 as data__room__pb2
from . import attestation_pb2 as attestation__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\tgcg.proto\x12\x03gcg\x1a\x0f\x64\x61ta_room.proto\x1a\x11\x61ttestation.proto\"\x86\x06\n\nGcgRequest\x12;\n\x15\x65xecuteComputeRequest\x18\x02 \x01(\x0b\x32\x1a.gcg.ExecuteComputeRequestH\x00\x12;\n\x15\x63reateDataRoomRequest\x18\x03 \x01(\x0b\x32\x1a.gcg.CreateDataRoomRequestH\x00\x12O\n\x1fpublishDatasetToDataRoomRequest\x18\x04 \x01(\x0b\x32$.gcg.PublishDatasetToDataRoomRequestH\x00\x12?\n\x17retrieveDataRoomRequest\x18\x05 \x01(\x0b\x32\x1c.gcg.RetrieveDataRoomRequestH\x00\x12?\n\x17retrieveAuditLogRequest\x18\x06 \x01(\x0b\x32\x1c.gcg.RetrieveAuditLogRequestH\x00\x12K\n\x1dretrieveDataRoomStatusRequest\x18\x07 \x01(\x0b\x32\".gcg.RetrieveDataRoomStatusRequestH\x00\x12G\n\x1bupdateDataRoomStatusRequest\x18\n \x01(\x0b\x32 .gcg.UpdateDataRoomStatusRequestH\x00\x12Q\n retrievePublishedDatasetsRequest\x18\x0b \x01(\x0b\x32%.gcg.RetrievePublishedDatasetsRequestH\x00\x12K\n\x1dremovePublishedDatasetRequest\x18\x0c \x01(\x0b\x32\".gcg.RemovePublishedDatasetRequestH\x00\x12\x31\n\x10jobStatusRequest\x18\r \x01(\x0b\x32\x15.gcg.JobStatusRequestH\x00\x12\x33\n\x11getResultsRequest\x18\x0e \x01(\x0b\x32\x16.gcg.GetResultsRequestH\x00\x42\r\n\x0bgcg_request\"\xfe\x06\n\x0bGcgResponse\x12\x11\n\x07\x66\x61ilure\x18\x01 \x01(\tH\x00\x12=\n\x16\x65xecuteComputeResponse\x18\x02 \x01(\x0b\x32\x1b.gcg.ExecuteComputeResponseH\x00\x12=\n\x16\x63reateDataRoomResponse\x18\x03 \x01(\x0b\x32\x1b.gcg.CreateDataRoomResponseH\x00\x12Q\n publishDatasetToDataRoomResponse\x18\x04 \x01(\x0b\x32%.gcg.PublishDatasetToDataRoomResponseH\x00\x12\x41\n\x18retrieveDataRoomResponse\x18\x05 \x01(\x0b\x32\x1d.gcg.RetrieveDataRoomResponseH\x00\x12\x41\n\x18retrieveAuditLogResponse\x18\x06 \x01(\x0b\x32\x1d.gcg.RetrieveAuditLogResponseH\x00\x12M\n\x1eretrieveDataRoomStatusResponse\x18\x07 \x01(\x0b\x32#.gcg.RetrieveDataRoomStatusResponseH\x00\x12I\n\x1cupdateDataRoomStatusResponse\x18\n \x01(\x0b\x32!.gcg.UpdateDataRoomStatusResponseH\x00\x12S\n!retrievePublishedDatasetsResponse\x18\x0b \x01(\x0b\x32&.gcg.RetrievePublishedDatasetsResponseH\x00\x12M\n\x1eremovePublishedDatasetResponse\x18\x0c \x01(\x0b\x32#.gcg.RemovePublishedDatasetResponseH\x00\x12\x33\n\x11jobStatusResponse\x18\r \x01(\x0b\x32\x16.gcg.JobStatusResponseH\x00\x12?\n\x17getResultsResponseChunk\x18\x0e \x01(\x0b\x32\x1c.gcg.GetResultsResponseChunkH\x00\x12\x41\n\x18getResultsResponseFooter\x18\x0f \x01(\x0b\x32\x1d.gcg.GetResultsResponseFooterH\x00\x42\x0e\n\x0cgcg_response\"M\n\x15\x43reateDataRoomRequest\x12%\n\x08\x64\x61taRoom\x18\x01 \x01(\x0b\x32\x13.data_room.DataRoom\x12\r\n\x05scope\x18\x02 \x01(\x0c\"\x8c\x01\n\x16\x43reateDataRoomResponse\x12\x14\n\ndataRoomId\x18\x01 \x01(\x0cH\x00\x12?\n\x17\x64\x61taRoomValidationError\x18\x02 \x01(\x0b\x32\x1c.gcg.DataRoomValidationErrorH\x00\x42\x1b\n\x19\x63reate_data_room_response\"\xb0\x03\n\x17\x44\x61taRoomValidationError\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\x1d\n\x10\x63omputeNodeIndex\x18\x02 \x01(\x04H\x00\x88\x01\x01\x12\x18\n\x0b\x63olumnIndex\x18\x03 \x01(\x04H\x01\x88\x01\x01\x12\x17\n\ntableIndex\x18\x04 \x01(\x04H\x02\x88\x01\x01\x12 \n\x13userPermissionIndex\x18\x05 \x01(\x04H\x03\x88\x01\x01\x12\x1c\n\x0fpermissionIndex\x18\x06 \x01(\x04H\x04\x88\x01\x01\x12*\n\x1d\x61ttestationSpecificationIndex\x18\x07 \x01(\x04H\x05\x88\x01\x01\x12&\n\x19\x61uthenticationMethodIndex\x18\x08 \x01(\x04H\x06\x88\x01\x01\x42\x13\n\x11_computeNodeIndexB\x0e\n\x0c_columnIndexB\r\n\x0b_tableIndexB\x16\n\x14_userPermissionIndexB\x12\n\x10_permissionIndexB \n\x1e_attestationSpecificationIndexB\x1c\n\x1a_authenticationMethodIndex\"\x82\x01\n\x1fPublishDatasetToDataRoomRequest\x12\x13\n\x0b\x64\x61tasetHash\x18\x01 \x01(\x0c\x12\x12\n\ndataRoomId\x18\x02 \x01(\x0c\x12\x10\n\x08leafName\x18\x03 \x01(\t\x12\x15\n\rencryptionKey\x18\x04 \x01(\x0c\x12\r\n\x05scope\x18\x05 \x01(\x0c\"\"\n PublishDatasetToDataRoomResponse\"f\n\x15\x45xecuteComputeRequest\x12\x12\n\ndataRoomId\x18\x01 \x01(\x0c\x12\x18\n\x10\x63omputeNodeNames\x18\x02 \x03(\t\x12\x10\n\x08isDryRun\x18\x03 \x01(\x08\x12\r\n\x05scope\x18\x04 \x01(\x0c\"\'\n\x16\x45xecuteComputeResponse\x12\r\n\x05jobId\x18\x01 \x01(\x0c\"!\n\x10JobStatusRequest\x12\r\n\x05jobId\x18\x01 \x01(\x0c\"5\n\x11JobStatusResponse\x12 \n\x18\x63ompleteComputeNodeNames\x18\x01 \x03(\t\";\n\x11GetResultsRequest\x12\r\n\x05jobId\x18\x01 \x01(\x0c\x12\x17\n\x0f\x63omputeNodeName\x18\x02 \x01(\t\"\'\n\x17GetResultsResponseChunk\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\"\x1a\n\x18GetResultsResponseFooter\"<\n\x17RetrieveDataRoomRequest\x12\x12\n\ndataRoomId\x18\x01 \x01(\x0c\x12\r\n\x05scope\x18\x02 \x01(\x0c\"A\n\x18RetrieveDataRoomResponse\x12%\n\x08\x64\x61taRoom\x18\x01 \x01(\x0b\x32\x13.data_room.DataRoom\"<\n\x17RetrieveAuditLogRequest\x12\x12\n\ndataRoomId\x18\x01 \x01(\x0c\x12\r\n\x05scope\x18\x02 \x01(\x0c\"\'\n\x18RetrieveAuditLogResponse\x12\x0b\n\x03log\x18\x01 \x01(\x0c\"B\n\x1dRetrieveDataRoomStatusRequest\x12\x12\n\ndataRoomId\x18\x01 \x01(\x0c\x12\r\n\x05scope\x18\x02 \x01(\x0c\"E\n\x1eRetrieveDataRoomStatusResponse\x12#\n\x06status\x18\x01 \x01(\x0e\x32\x13.gcg.DataRoomStatus\"e\n\x1bUpdateDataRoomStatusRequest\x12\x12\n\ndataRoomId\x18\x01 \x01(\x0c\x12\r\n\x05scope\x18\x02 \x01(\x0c\x12#\n\x06status\x18\x03 \x01(\x0e\x32\x13.gcg.DataRoomStatus\"\x1e\n\x1cUpdateDataRoomStatusResponse\"E\n RetrievePublishedDatasetsRequest\x12\x12\n\ndataRoomId\x18\x01 \x01(\x0c\x12\r\n\x05scope\x18\x02 \x01(\x0c\"Z\n\x10PublishedDataset\x12\x10\n\x08leafName\x18\x01 \x01(\t\x12\x0c\n\x04user\x18\x02 \x01(\t\x12\x11\n\ttimestamp\x18\x03 \x01(\x04\x12\x13\n\x0b\x64\x61tasetHash\x18\x04 \x01(\x0c\"U\n!RetrievePublishedDatasetsResponse\x12\x30\n\x11publishedDatasets\x18\x01 \x03(\x0b\x32\x15.gcg.PublishedDataset\"Z\n\x1dRemovePublishedDatasetRequest\x12\x12\n\ndataRoomId\x18\x02 \x01(\x0c\x12\x10\n\x08leafName\x18\x03 \x01(\t\x12\r\n\x05scope\x18\x04 \x01(\x0cJ\x04\x08\x01\x10\x02\" \n\x1eRemovePublishedDatasetResponse\"|\n\x10\x44riverTaskConfig\x12\x1f\n\x04noop\x18\x01 \x01(\x0b\x32\x0f.gcg.NoopConfigH\x00\x12\x31\n\rstaticContent\x18\x02 \x01(\x0b\x32\x18.gcg.StaticContentConfigH\x00\x42\x14\n\x12\x64river_task_config\"\x0c\n\nNoopConfig\"&\n\x13StaticContentConfig\x12\x0f\n\x07\x63ontent\x18\x01 \x01(\x0c*)\n\x0e\x44\x61taRoomStatus\x12\n\n\x06\x41\x63tive\x10\x00\x12\x0b\n\x07Stopped\x10\x01\x62\x06proto3')

_DATAROOMSTATUS = DESCRIPTOR.enum_types_by_name['DataRoomStatus']
DataRoomStatus = enum_type_wrapper.EnumTypeWrapper(_DATAROOMSTATUS)
Active = 0
Stopped = 1


_GCGREQUEST = DESCRIPTOR.message_types_by_name['GcgRequest']
_GCGRESPONSE = DESCRIPTOR.message_types_by_name['GcgResponse']
_CREATEDATAROOMREQUEST = DESCRIPTOR.message_types_by_name['CreateDataRoomRequest']
_CREATEDATAROOMRESPONSE = DESCRIPTOR.message_types_by_name['CreateDataRoomResponse']
_DATAROOMVALIDATIONERROR = DESCRIPTOR.message_types_by_name['DataRoomValidationError']
_PUBLISHDATASETTODATAROOMREQUEST = DESCRIPTOR.message_types_by_name['PublishDatasetToDataRoomRequest']
_PUBLISHDATASETTODATAROOMRESPONSE = DESCRIPTOR.message_types_by_name['PublishDatasetToDataRoomResponse']
_EXECUTECOMPUTEREQUEST = DESCRIPTOR.message_types_by_name['ExecuteComputeRequest']
_EXECUTECOMPUTERESPONSE = DESCRIPTOR.message_types_by_name['ExecuteComputeResponse']
_JOBSTATUSREQUEST = DESCRIPTOR.message_types_by_name['JobStatusRequest']
_JOBSTATUSRESPONSE = DESCRIPTOR.message_types_by_name['JobStatusResponse']
_GETRESULTSREQUEST = DESCRIPTOR.message_types_by_name['GetResultsRequest']
_GETRESULTSRESPONSECHUNK = DESCRIPTOR.message_types_by_name['GetResultsResponseChunk']
_GETRESULTSRESPONSEFOOTER = DESCRIPTOR.message_types_by_name['GetResultsResponseFooter']
_RETRIEVEDATAROOMREQUEST = DESCRIPTOR.message_types_by_name['RetrieveDataRoomRequest']
_RETRIEVEDATAROOMRESPONSE = DESCRIPTOR.message_types_by_name['RetrieveDataRoomResponse']
_RETRIEVEAUDITLOGREQUEST = DESCRIPTOR.message_types_by_name['RetrieveAuditLogRequest']
_RETRIEVEAUDITLOGRESPONSE = DESCRIPTOR.message_types_by_name['RetrieveAuditLogResponse']
_RETRIEVEDATAROOMSTATUSREQUEST = DESCRIPTOR.message_types_by_name['RetrieveDataRoomStatusRequest']
_RETRIEVEDATAROOMSTATUSRESPONSE = DESCRIPTOR.message_types_by_name['RetrieveDataRoomStatusResponse']
_UPDATEDATAROOMSTATUSREQUEST = DESCRIPTOR.message_types_by_name['UpdateDataRoomStatusRequest']
_UPDATEDATAROOMSTATUSRESPONSE = DESCRIPTOR.message_types_by_name['UpdateDataRoomStatusResponse']
_RETRIEVEPUBLISHEDDATASETSREQUEST = DESCRIPTOR.message_types_by_name['RetrievePublishedDatasetsRequest']
_PUBLISHEDDATASET = DESCRIPTOR.message_types_by_name['PublishedDataset']
_RETRIEVEPUBLISHEDDATASETSRESPONSE = DESCRIPTOR.message_types_by_name['RetrievePublishedDatasetsResponse']
_REMOVEPUBLISHEDDATASETREQUEST = DESCRIPTOR.message_types_by_name['RemovePublishedDatasetRequest']
_REMOVEPUBLISHEDDATASETRESPONSE = DESCRIPTOR.message_types_by_name['RemovePublishedDatasetResponse']
_DRIVERTASKCONFIG = DESCRIPTOR.message_types_by_name['DriverTaskConfig']
_NOOPCONFIG = DESCRIPTOR.message_types_by_name['NoopConfig']
_STATICCONTENTCONFIG = DESCRIPTOR.message_types_by_name['StaticContentConfig']
GcgRequest = _reflection.GeneratedProtocolMessageType('GcgRequest', (_message.Message,), {
  'DESCRIPTOR' : _GCGREQUEST,
  '__module__' : 'gcg_pb2'
  # @@protoc_insertion_point(class_scope:gcg.GcgRequest)
  })
_sym_db.RegisterMessage(GcgRequest)

GcgResponse = _reflection.GeneratedProtocolMessageType('GcgResponse', (_message.Message,), {
  'DESCRIPTOR' : _GCGRESPONSE,
  '__module__' : 'gcg_pb2'
  # @@protoc_insertion_point(class_scope:gcg.GcgResponse)
  })
_sym_db.RegisterMessage(GcgResponse)

CreateDataRoomRequest = _reflection.GeneratedProtocolMessageType('CreateDataRoomRequest', (_message.Message,), {
  'DESCRIPTOR' : _CREATEDATAROOMREQUEST,
  '__module__' : 'gcg_pb2'
  # @@protoc_insertion_point(class_scope:gcg.CreateDataRoomRequest)
  })
_sym_db.RegisterMessage(CreateDataRoomRequest)

CreateDataRoomResponse = _reflection.GeneratedProtocolMessageType('CreateDataRoomResponse', (_message.Message,), {
  'DESCRIPTOR' : _CREATEDATAROOMRESPONSE,
  '__module__' : 'gcg_pb2'
  # @@protoc_insertion_point(class_scope:gcg.CreateDataRoomResponse)
  })
_sym_db.RegisterMessage(CreateDataRoomResponse)

DataRoomValidationError = _reflection.GeneratedProtocolMessageType('DataRoomValidationError', (_message.Message,), {
  'DESCRIPTOR' : _DATAROOMVALIDATIONERROR,
  '__module__' : 'gcg_pb2'
  # @@protoc_insertion_point(class_scope:gcg.DataRoomValidationError)
  })
_sym_db.RegisterMessage(DataRoomValidationError)

PublishDatasetToDataRoomRequest = _reflection.GeneratedProtocolMessageType('PublishDatasetToDataRoomRequest', (_message.Message,), {
  'DESCRIPTOR' : _PUBLISHDATASETTODATAROOMREQUEST,
  '__module__' : 'gcg_pb2'
  # @@protoc_insertion_point(class_scope:gcg.PublishDatasetToDataRoomRequest)
  })
_sym_db.RegisterMessage(PublishDatasetToDataRoomRequest)

PublishDatasetToDataRoomResponse = _reflection.GeneratedProtocolMessageType('PublishDatasetToDataRoomResponse', (_message.Message,), {
  'DESCRIPTOR' : _PUBLISHDATASETTODATAROOMRESPONSE,
  '__module__' : 'gcg_pb2'
  # @@protoc_insertion_point(class_scope:gcg.PublishDatasetToDataRoomResponse)
  })
_sym_db.RegisterMessage(PublishDatasetToDataRoomResponse)

ExecuteComputeRequest = _reflection.GeneratedProtocolMessageType('ExecuteComputeRequest', (_message.Message,), {
  'DESCRIPTOR' : _EXECUTECOMPUTEREQUEST,
  '__module__' : 'gcg_pb2'
  # @@protoc_insertion_point(class_scope:gcg.ExecuteComputeRequest)
  })
_sym_db.RegisterMessage(ExecuteComputeRequest)

ExecuteComputeResponse = _reflection.GeneratedProtocolMessageType('ExecuteComputeResponse', (_message.Message,), {
  'DESCRIPTOR' : _EXECUTECOMPUTERESPONSE,
  '__module__' : 'gcg_pb2'
  # @@protoc_insertion_point(class_scope:gcg.ExecuteComputeResponse)
  })
_sym_db.RegisterMessage(ExecuteComputeResponse)

JobStatusRequest = _reflection.GeneratedProtocolMessageType('JobStatusRequest', (_message.Message,), {
  'DESCRIPTOR' : _JOBSTATUSREQUEST,
  '__module__' : 'gcg_pb2'
  # @@protoc_insertion_point(class_scope:gcg.JobStatusRequest)
  })
_sym_db.RegisterMessage(JobStatusRequest)

JobStatusResponse = _reflection.GeneratedProtocolMessageType('JobStatusResponse', (_message.Message,), {
  'DESCRIPTOR' : _JOBSTATUSRESPONSE,
  '__module__' : 'gcg_pb2'
  # @@protoc_insertion_point(class_scope:gcg.JobStatusResponse)
  })
_sym_db.RegisterMessage(JobStatusResponse)

GetResultsRequest = _reflection.GeneratedProtocolMessageType('GetResultsRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETRESULTSREQUEST,
  '__module__' : 'gcg_pb2'
  # @@protoc_insertion_point(class_scope:gcg.GetResultsRequest)
  })
_sym_db.RegisterMessage(GetResultsRequest)

GetResultsResponseChunk = _reflection.GeneratedProtocolMessageType('GetResultsResponseChunk', (_message.Message,), {
  'DESCRIPTOR' : _GETRESULTSRESPONSECHUNK,
  '__module__' : 'gcg_pb2'
  # @@protoc_insertion_point(class_scope:gcg.GetResultsResponseChunk)
  })
_sym_db.RegisterMessage(GetResultsResponseChunk)

GetResultsResponseFooter = _reflection.GeneratedProtocolMessageType('GetResultsResponseFooter', (_message.Message,), {
  'DESCRIPTOR' : _GETRESULTSRESPONSEFOOTER,
  '__module__' : 'gcg_pb2'
  # @@protoc_insertion_point(class_scope:gcg.GetResultsResponseFooter)
  })
_sym_db.RegisterMessage(GetResultsResponseFooter)

RetrieveDataRoomRequest = _reflection.GeneratedProtocolMessageType('RetrieveDataRoomRequest', (_message.Message,), {
  'DESCRIPTOR' : _RETRIEVEDATAROOMREQUEST,
  '__module__' : 'gcg_pb2'
  # @@protoc_insertion_point(class_scope:gcg.RetrieveDataRoomRequest)
  })
_sym_db.RegisterMessage(RetrieveDataRoomRequest)

RetrieveDataRoomResponse = _reflection.GeneratedProtocolMessageType('RetrieveDataRoomResponse', (_message.Message,), {
  'DESCRIPTOR' : _RETRIEVEDATAROOMRESPONSE,
  '__module__' : 'gcg_pb2'
  # @@protoc_insertion_point(class_scope:gcg.RetrieveDataRoomResponse)
  })
_sym_db.RegisterMessage(RetrieveDataRoomResponse)

RetrieveAuditLogRequest = _reflection.GeneratedProtocolMessageType('RetrieveAuditLogRequest', (_message.Message,), {
  'DESCRIPTOR' : _RETRIEVEAUDITLOGREQUEST,
  '__module__' : 'gcg_pb2'
  # @@protoc_insertion_point(class_scope:gcg.RetrieveAuditLogRequest)
  })
_sym_db.RegisterMessage(RetrieveAuditLogRequest)

RetrieveAuditLogResponse = _reflection.GeneratedProtocolMessageType('RetrieveAuditLogResponse', (_message.Message,), {
  'DESCRIPTOR' : _RETRIEVEAUDITLOGRESPONSE,
  '__module__' : 'gcg_pb2'
  # @@protoc_insertion_point(class_scope:gcg.RetrieveAuditLogResponse)
  })
_sym_db.RegisterMessage(RetrieveAuditLogResponse)

RetrieveDataRoomStatusRequest = _reflection.GeneratedProtocolMessageType('RetrieveDataRoomStatusRequest', (_message.Message,), {
  'DESCRIPTOR' : _RETRIEVEDATAROOMSTATUSREQUEST,
  '__module__' : 'gcg_pb2'
  # @@protoc_insertion_point(class_scope:gcg.RetrieveDataRoomStatusRequest)
  })
_sym_db.RegisterMessage(RetrieveDataRoomStatusRequest)

RetrieveDataRoomStatusResponse = _reflection.GeneratedProtocolMessageType('RetrieveDataRoomStatusResponse', (_message.Message,), {
  'DESCRIPTOR' : _RETRIEVEDATAROOMSTATUSRESPONSE,
  '__module__' : 'gcg_pb2'
  # @@protoc_insertion_point(class_scope:gcg.RetrieveDataRoomStatusResponse)
  })
_sym_db.RegisterMessage(RetrieveDataRoomStatusResponse)

UpdateDataRoomStatusRequest = _reflection.GeneratedProtocolMessageType('UpdateDataRoomStatusRequest', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEDATAROOMSTATUSREQUEST,
  '__module__' : 'gcg_pb2'
  # @@protoc_insertion_point(class_scope:gcg.UpdateDataRoomStatusRequest)
  })
_sym_db.RegisterMessage(UpdateDataRoomStatusRequest)

UpdateDataRoomStatusResponse = _reflection.GeneratedProtocolMessageType('UpdateDataRoomStatusResponse', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEDATAROOMSTATUSRESPONSE,
  '__module__' : 'gcg_pb2'
  # @@protoc_insertion_point(class_scope:gcg.UpdateDataRoomStatusResponse)
  })
_sym_db.RegisterMessage(UpdateDataRoomStatusResponse)

RetrievePublishedDatasetsRequest = _reflection.GeneratedProtocolMessageType('RetrievePublishedDatasetsRequest', (_message.Message,), {
  'DESCRIPTOR' : _RETRIEVEPUBLISHEDDATASETSREQUEST,
  '__module__' : 'gcg_pb2'
  # @@protoc_insertion_point(class_scope:gcg.RetrievePublishedDatasetsRequest)
  })
_sym_db.RegisterMessage(RetrievePublishedDatasetsRequest)

PublishedDataset = _reflection.GeneratedProtocolMessageType('PublishedDataset', (_message.Message,), {
  'DESCRIPTOR' : _PUBLISHEDDATASET,
  '__module__' : 'gcg_pb2'
  # @@protoc_insertion_point(class_scope:gcg.PublishedDataset)
  })
_sym_db.RegisterMessage(PublishedDataset)

RetrievePublishedDatasetsResponse = _reflection.GeneratedProtocolMessageType('RetrievePublishedDatasetsResponse', (_message.Message,), {
  'DESCRIPTOR' : _RETRIEVEPUBLISHEDDATASETSRESPONSE,
  '__module__' : 'gcg_pb2'
  # @@protoc_insertion_point(class_scope:gcg.RetrievePublishedDatasetsResponse)
  })
_sym_db.RegisterMessage(RetrievePublishedDatasetsResponse)

RemovePublishedDatasetRequest = _reflection.GeneratedProtocolMessageType('RemovePublishedDatasetRequest', (_message.Message,), {
  'DESCRIPTOR' : _REMOVEPUBLISHEDDATASETREQUEST,
  '__module__' : 'gcg_pb2'
  # @@protoc_insertion_point(class_scope:gcg.RemovePublishedDatasetRequest)
  })
_sym_db.RegisterMessage(RemovePublishedDatasetRequest)

RemovePublishedDatasetResponse = _reflection.GeneratedProtocolMessageType('RemovePublishedDatasetResponse', (_message.Message,), {
  'DESCRIPTOR' : _REMOVEPUBLISHEDDATASETRESPONSE,
  '__module__' : 'gcg_pb2'
  # @@protoc_insertion_point(class_scope:gcg.RemovePublishedDatasetResponse)
  })
_sym_db.RegisterMessage(RemovePublishedDatasetResponse)

DriverTaskConfig = _reflection.GeneratedProtocolMessageType('DriverTaskConfig', (_message.Message,), {
  'DESCRIPTOR' : _DRIVERTASKCONFIG,
  '__module__' : 'gcg_pb2'
  # @@protoc_insertion_point(class_scope:gcg.DriverTaskConfig)
  })
_sym_db.RegisterMessage(DriverTaskConfig)

NoopConfig = _reflection.GeneratedProtocolMessageType('NoopConfig', (_message.Message,), {
  'DESCRIPTOR' : _NOOPCONFIG,
  '__module__' : 'gcg_pb2'
  # @@protoc_insertion_point(class_scope:gcg.NoopConfig)
  })
_sym_db.RegisterMessage(NoopConfig)

StaticContentConfig = _reflection.GeneratedProtocolMessageType('StaticContentConfig', (_message.Message,), {
  'DESCRIPTOR' : _STATICCONTENTCONFIG,
  '__module__' : 'gcg_pb2'
  # @@protoc_insertion_point(class_scope:gcg.StaticContentConfig)
  })
_sym_db.RegisterMessage(StaticContentConfig)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _DATAROOMSTATUS._serialized_start=3981
  _DATAROOMSTATUS._serialized_end=4022
  _GCGREQUEST._serialized_start=55
  _GCGREQUEST._serialized_end=829
  _GCGRESPONSE._serialized_start=832
  _GCGRESPONSE._serialized_end=1726
  _CREATEDATAROOMREQUEST._serialized_start=1728
  _CREATEDATAROOMREQUEST._serialized_end=1805
  _CREATEDATAROOMRESPONSE._serialized_start=1808
  _CREATEDATAROOMRESPONSE._serialized_end=1948
  _DATAROOMVALIDATIONERROR._serialized_start=1951
  _DATAROOMVALIDATIONERROR._serialized_end=2383
  _PUBLISHDATASETTODATAROOMREQUEST._serialized_start=2386
  _PUBLISHDATASETTODATAROOMREQUEST._serialized_end=2516
  _PUBLISHDATASETTODATAROOMRESPONSE._serialized_start=2518
  _PUBLISHDATASETTODATAROOMRESPONSE._serialized_end=2552
  _EXECUTECOMPUTEREQUEST._serialized_start=2554
  _EXECUTECOMPUTEREQUEST._serialized_end=2656
  _EXECUTECOMPUTERESPONSE._serialized_start=2658
  _EXECUTECOMPUTERESPONSE._serialized_end=2697
  _JOBSTATUSREQUEST._serialized_start=2699
  _JOBSTATUSREQUEST._serialized_end=2732
  _JOBSTATUSRESPONSE._serialized_start=2734
  _JOBSTATUSRESPONSE._serialized_end=2787
  _GETRESULTSREQUEST._serialized_start=2789
  _GETRESULTSREQUEST._serialized_end=2848
  _GETRESULTSRESPONSECHUNK._serialized_start=2850
  _GETRESULTSRESPONSECHUNK._serialized_end=2889
  _GETRESULTSRESPONSEFOOTER._serialized_start=2891
  _GETRESULTSRESPONSEFOOTER._serialized_end=2917
  _RETRIEVEDATAROOMREQUEST._serialized_start=2919
  _RETRIEVEDATAROOMREQUEST._serialized_end=2979
  _RETRIEVEDATAROOMRESPONSE._serialized_start=2981
  _RETRIEVEDATAROOMRESPONSE._serialized_end=3046
  _RETRIEVEAUDITLOGREQUEST._serialized_start=3048
  _RETRIEVEAUDITLOGREQUEST._serialized_end=3108
  _RETRIEVEAUDITLOGRESPONSE._serialized_start=3110
  _RETRIEVEAUDITLOGRESPONSE._serialized_end=3149
  _RETRIEVEDATAROOMSTATUSREQUEST._serialized_start=3151
  _RETRIEVEDATAROOMSTATUSREQUEST._serialized_end=3217
  _RETRIEVEDATAROOMSTATUSRESPONSE._serialized_start=3219
  _RETRIEVEDATAROOMSTATUSRESPONSE._serialized_end=3288
  _UPDATEDATAROOMSTATUSREQUEST._serialized_start=3290
  _UPDATEDATAROOMSTATUSREQUEST._serialized_end=3391
  _UPDATEDATAROOMSTATUSRESPONSE._serialized_start=3393
  _UPDATEDATAROOMSTATUSRESPONSE._serialized_end=3423
  _RETRIEVEPUBLISHEDDATASETSREQUEST._serialized_start=3425
  _RETRIEVEPUBLISHEDDATASETSREQUEST._serialized_end=3494
  _PUBLISHEDDATASET._serialized_start=3496
  _PUBLISHEDDATASET._serialized_end=3586
  _RETRIEVEPUBLISHEDDATASETSRESPONSE._serialized_start=3588
  _RETRIEVEPUBLISHEDDATASETSRESPONSE._serialized_end=3673
  _REMOVEPUBLISHEDDATASETREQUEST._serialized_start=3675
  _REMOVEPUBLISHEDDATASETREQUEST._serialized_end=3765
  _REMOVEPUBLISHEDDATASETRESPONSE._serialized_start=3767
  _REMOVEPUBLISHEDDATASETRESPONSE._serialized_end=3799
  _DRIVERTASKCONFIG._serialized_start=3801
  _DRIVERTASKCONFIG._serialized_end=3925
  _NOOPCONFIG._serialized_start=3927
  _NOOPCONFIG._serialized_end=3939
  _STATICCONTENTCONFIG._serialized_start=3941
  _STATICCONTENTCONFIG._serialized_end=3979
# @@protoc_insertion_point(module_scope)



from __future__ import annotations
from typing import List, Any, Optional, Callable, Dict
from .proto import (
    AuthenticationMethod, ComputeNodeFormat, UserPermission, ComputeNode,
    ComputeNodeLeaf, ComputeNodeBranch, Permission, DataRoom,
    AttestationSpecification
)
from .types import EnclaveSpecification
import random


class DataRoomBuilder():
    """
    An helper class to ease the building process of a data clean room
    """
    attestation_specifications: List[AttestationSpecification]
    authentication_methods: List[AuthenticationMethod]
    user_permissions: List[UserPermission]
    compute_nodes: List[ComputeNode]
    id: Optional[str]
    name: str
    description: Optional[str]
    owner_email: Optional[str]
    enclave_specs: Optional[Dict[str, EnclaveSpecification]]

    def __init__(
            self,
            name: str,
            enclave_specs: Optional[Dict[str, EnclaveSpecification]] = None,
            *,
            description: str = None,
            owner_email: str = None,
        ) -> None:
        assert name, "The DCR must have a non-empty name"

        self.name = name
        self.attestation_specifications = list()
        self.authentication_methods = list()
        self.user_permissions = list()
        self.compute_nodes = list()
        self.owner_email = owner_email
        self.description = description
        self.id = None
        self.enclave_specs = enclave_specs

        if description:
            self.add_description(description)

    def using_enclave_specs(self, enclave_specs: Dict[str, EnclaveSpecification]):
        """
        Configure the builder to use the given set of enclave specs. Whenever a new compute
        node is added, the builder will inspect the set of enclave specs to find the correct
        spec that matches the enclave type required by the compute node.
        This method should be called before any compute nodes are added.
        """
        self.enclave_specs = enclave_specs

    def add_description(self, description: str):
        """Add a description to the data room being built."""
        self.description = description

    def add_owner_email(self, email: str):
        """Specify the owner of the data room."""
        self.owner_email = email

    def add_data_node(self, name: str, is_required=False):
        """
        Add a new data node. If the node is marked as required, any computation
        which includes it as a dependency will not progress.
        """
        node = ComputeNode(
            nodeName=name,
            leaf=ComputeNodeLeaf(isRequired=is_required)
        )
        self.compute_nodes.append(node)

    def add_using_function(self, do_with_builder: Callable[[DataRoomBuilder], None]):
        """Apply the given function to the builder, transforming it in useful ways."""
        do_with_builder(self)

    def add_compute_node(
            self,
            name: str,
            node: Any,
            dependencies: List[str] = []
    ):
        """
        Add a new compute node. The configuration for a compute node should be
        created using one of the compute-specific libraries.
        """

        required_attributes = set(["enclave_type", "config", "output_format"])
        if not all([hasattr(node, attr) for attr in required_attributes]):
            raise Exception(
                "The given node object is missing a required attribute"
                f" (required: {', '.join(required_attributes)}). "
                "Are you sure that this is a proper node object?"
                " Refer to the documentation of the class you're trying to pass in "
                " to see how it should be used."
            )

        if self.enclave_specs:
            if node.enclave_type in self.enclave_specs:
                attestation_proto = self.enclave_specs[node.enclave_type]["proto"]
            else:
                raise Exception(
                    f"This compute node requires an enclave of type '{node.enclave_type}' but no"
                    " corresponding attestation spec was provided to this builder."
                )
        else:
            raise Exception(
                "You need to provide an attestation spec set to the builder"
                " before calling this method."
            )
        try:
            attestation_index = self.attestation_specifications.index(attestation_proto)
        except ValueError:
            self.attestation_specifications.append(attestation_proto)
            attestation_index = len(self.attestation_specifications) - 1
        proto_node = ComputeNode(
            nodeName=name,
            branch=ComputeNodeBranch(
                config=node.config,
                attestationSpecificationIndex=attestation_index,
                dependencies=dependencies,
                outputFormat=node.output_format,
            )
        )
        self.compute_nodes.append(proto_node)

    def add_user_permission(
            self,
            email: str,
            authentication_method: AuthenticationMethod,
            permissions: List[Permission],
    ):
        """
        Add permissions for a user. The authentication is performed on the enclave side
        based on the method supplied.
        """
        try:
            authentication_method_index = self.authentication_methods.index(authentication_method)
        except ValueError:
            self.authentication_methods.append(authentication_method)
            authentication_method_index = len(self.authentication_methods) - 1

        permission = UserPermission(
            email=email,
            authenticationMethodIndex=authentication_method_index,
            permissions=permissions
        )
        self.user_permissions.append(permission)

    def set_id(self, id: str):
        """
        Set the **internal** identifier for the data room.

        **This is not used for addressing the data room**.
        When publishing the data room, you will recieve the data room hash that can be used
        to interact with the data room.
        By default this internal id will be automatically randomly generated and therefore calling
        this method is very likely not necessary.

        This values main purpose is to give additional entropy to the dataroom definition so that
        otherwise equivalent dataroom definitions may be differentiated.
        It may also be used for retries to ensure exactly-once creation of the dataroom.
        """
        self.id = id

    def build(self) -> DataRoom:
        """
        Finalize data room contruction.

        Built data rooms still need to be published by a `decentriq_platform.Session` before they can
        be interacted with.
        """
        data_room = DataRoom()
        data_room.name = self.name
        data_room.computeNodes.extend(self.compute_nodes)
        data_room.attestationSpecifications.extend(self.attestation_specifications)
        data_room.userPermissions.extend(self.user_permissions)
        data_room.authenticationMethods.extend(self.authentication_methods)

        if not self.owner_email:
            raise Exception("The data room requires an owner email address to bet set!")
        else:
            data_room.ownerEmail = self.owner_email

        if self.id:
            data_room.id = self.id
        else:
            data_room.id = random.getrandbits(64).to_bytes(8, byteorder='little').hex()

        if self.description:
            data_room.description = self.description

        return data_room



"""
  include:: ../../../decentriq_platform_docs/sql_getting_started.md
---
"""
__docformat__ = "restructuredtext"

from .compute import SqlCompute, SqlSchemaVerifier
from .proto.compute_sql_pb2 import (
    PrimitiveType,
)
from .attestation import EnclaveSpecifications
from .helpers import (
    ValidatedDataNode,
    read_sql_query_result_as_string,
    read_input_csv_file,
    read_input_csv_string,
    upload_and_publish_dataset,
)


__all__ = [
    "SqlCompute",
    "ValidatedDataNode",
    "PrimitiveType",
    "read_input_csv_file",
    "read_input_csv_string",
    "upload_and_publish_dataset",
    "read_sql_query_result_as_string",
    "EnclaveSpecifications",
    "SqlSchemaVerifier",
]



from .data_room_pb2 import (
    Permission,
    LeafCrudPermission,
    RetrieveAuditLogPermission,
    ExecuteComputePermission,
    RetrieveDataRoomPermission,
    RetrieveDataRoomStatusPermission,
    UpdateDataRoomStatusPermission,
    RetrievePublishedDatasetsPermission,
    DryRunPermission,
    AuthenticationMethod, UserPermission, ComputeNode,
    ComputeNodeLeaf, ComputeNodeBranch, DataRoom,
    TrustedPki,
    ComputeNodeFormat,
)
from .attestation_pb2 import (
    AttestationSpecification,
    AttestationSpecificationIntelEpid,
    Fatquote,
    FatquoteEpid, FatquoteDcap,
    AttestationSpecificationIntelDcap,
)
from .gcg_pb2 import (
    DriverTaskConfig, NoopConfig, DataRoomStatus,
    CreateDataRoomRequest, CreateDataRoomResponse,
    ExecuteComputeRequest, ExecuteComputeResponse, GcgRequest, GcgResponse, GetResultsRequest,
    GetResultsResponseChunk, JobStatusRequest, JobStatusResponse,
    PublishDatasetToDataRoomRequest, PublishDatasetToDataRoomResponse,
    RemovePublishedDatasetRequest, RemovePublishedDatasetResponse,
    RetrieveAuditLogRequest, RetrieveAuditLogResponse, RetrieveDataRoomRequest,
    RetrieveDataRoomResponse, RetrieveDataRoomStatusRequest,
    RetrieveDataRoomStatusResponse, RetrievePublishedDatasetsRequest,
    RetrievePublishedDatasetsResponse, UpdateDataRoomStatusRequest,
    UpdateDataRoomStatusResponse,
    StaticContentConfig,
)
from .length_delimited import parse_length_delimited, serialize_length_delimited
from .delta_enclave_api_pb2 import (
    ChunkHeader, EncryptionHeader, VersionHeader,
    DataNoncePubkey, Request, Response
)




from __future__ import annotations
import sqloxide
from typing import List, Optional, Set, Tuple
from .proto import (
    PrimitiveType, PrivacySettings, ComputationConfiguration, TableSchema,
    ValidationConfiguration, Constraint, NamedColumn, ColumnType, SqlWorkerConfiguration,
)
from ..proto import serialize_length_delimited, ComputeNodeFormat


def parse_statement(sql_statement: str):
    statements = sqloxide.parse_sql(sql=sql_statement, dialect='generic')
    if len(statements) != 1:
        raise Exception("Only 1 statment allowed in SQL string")

    statement = statements[0]
    if "Query" not in statement:
        raise Exception("Non-query statement in SQL string")
    return statement["Query"]


def find_referenced_tables_table_factor(table_factor) -> Set[str]:
    if "Table" in table_factor:
        a = {table_factor["Table"]["name"][0]["value"]}
        return a
    elif "Derived" in table_factor:
        return find_referenced_tables(table_factor["Derived"]["subquery"])
    else:
        raise Exception("Unsupported SQL table factor")


def find_referenced_tables_body(sql_body) -> Set[str]:
    if "Select" in sql_body:
        from_select_statements = sql_body["Select"]["from"]
        if len(from_select_statements) != 1:
            raise Exception("Invalid FROM SQL statement")
        from_statement = from_select_statements[0]

        join_referenced_tables: Set[str] = set()
        for join_statement in from_statement["joins"]:
            join_referenced_tables.update(
                find_referenced_tables_table_factor(join_statement["relation"])
            )

        table_factor_referenced_tables = find_referenced_tables_table_factor(from_statement["relation"])
        return join_referenced_tables.union(table_factor_referenced_tables)
    elif "SetOperation" in sql_body:
        set_operation_statement = sql_body["SetOperation"]
        left_statement = set_operation_statement["left"]
        right_statement = set_operation_statement["right"]
        left_referenced_tables = find_referenced_tables_body(left_statement)
        right_referenced_tables = find_referenced_tables_body(right_statement)
        return left_referenced_tables.union(right_referenced_tables)
    else:
        raise Exception("Unsupported SQL expression")


def find_referenced_tables(sql_query) -> Set[str]:
    cte_tables: Set[str] = set()
    cte_referenced_tables: Set[str] = set()
    with_statement = sql_query["with"]
    if with_statement:
        cte_tables_statements = sql_query["with"]["cte_tables"]
        for table in cte_tables_statements:
            cte_tables.add(table["alias"]["name"]["value"])
            cte_referenced_tables.update(find_referenced_tables(table["query"]))

    query_tables = find_referenced_tables_body(sql_query["body"])
    all_tables = query_tables.union(cte_referenced_tables)
    return all_tables.difference(cte_tables)


class SqlCompute():
    """
    Computation node to execute an SQL query
    """
    config: bytes
    """Serialized configuration to use in the compute node definition"""
    dependencies: List[str]
    """Dependencies automatically extracted from the SQL query"""

    def __init__(
            self,
            sql_statement: str,
            privacy_settings: Optional[PrivacySettings] = None,
            constraints: Optional[List[Constraint]] = None,
    ) -> None:
        statement_ast = parse_statement(sql_statement)
        self.dependencies = list(find_referenced_tables(statement_ast))

        sql_worker_configuration = SqlWorkerConfiguration(
            computation=ComputationConfiguration(
                sqlStatement=sql_statement,
                privacySettings=privacy_settings,
                constraints=constraints

            )
        )
        self.config = serialize_length_delimited(sql_worker_configuration)
        self.enclave_type = "decentriq.sql-worker"
        self.output_format = ComputeNodeFormat.ZIP


class SqlSchemaVerifier():
    """
    Computation node to validate an input and provide the necessary types
    """
    config: bytes
    """Serialized configuration to use in the compute node definition"""
    config: bytes

    def __init__(
        self,
        columns # type: List[Tuple[str, PrimitiveType.V, bool]]
    ) -> None:
        named_columns = map(lambda c: NamedColumn(
            name=c[0],
            columnType=ColumnType(
                primitiveType=c[1],
                nullable=c[2]
                )
            ),
            columns
        )
        sql_worker_configuration = SqlWorkerConfiguration(
            validation=ValidationConfiguration(
                tableSchema=TableSchema(namedColumns=named_columns)
            )
        )
        self.config = serialize_length_delimited(sql_worker_configuration)
        self.enclave_type = "decentriq.sql-worker"
        self.output_format = ComputeNodeFormat.ZIP



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor = ...

class ContainerWorkerConfiguration(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    STATIC_FIELD_NUMBER: builtins.int
    @property
    def static(self) -> global___StaticImage: ...
    def __init__(self,
        *,
        static : typing.Optional[global___StaticImage] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"configuration",b"configuration",u"static",b"static"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"configuration",b"configuration",u"static",b"static"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal[u"configuration",b"configuration"]) -> typing.Optional[typing_extensions.Literal["static"]]: ...
global___ContainerWorkerConfiguration = ContainerWorkerConfiguration

class StaticImage(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    COMMAND_FIELD_NUMBER: builtins.int
    MOUNTPOINTS_FIELD_NUMBER: builtins.int
    OUTPUTPATH_FIELD_NUMBER: builtins.int
    INCLUDECONTAINERLOGSONERROR_FIELD_NUMBER: builtins.int
    @property
    def command(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[typing.Text]: ...
    @property
    def mountPoints(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___MountPoint]: ...
    outputPath: typing.Text = ...
    includeContainerLogsOnError: builtins.bool = ...
    def __init__(self,
        *,
        command : typing.Optional[typing.Iterable[typing.Text]] = ...,
        mountPoints : typing.Optional[typing.Iterable[global___MountPoint]] = ...,
        outputPath : typing.Text = ...,
        includeContainerLogsOnError : builtins.bool = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"command",b"command",u"includeContainerLogsOnError",b"includeContainerLogsOnError",u"mountPoints",b"mountPoints",u"outputPath",b"outputPath"]) -> None: ...
global___StaticImage = StaticImage

class MountPoint(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    PATH_FIELD_NUMBER: builtins.int
    DEPENDENCY_FIELD_NUMBER: builtins.int
    path: typing.Text = ...
    dependency: typing.Text = ...
    def __init__(self,
        *,
        path : typing.Text = ...,
        dependency : typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"dependency",b"dependency",u"path",b"path"]) -> None: ...
global___MountPoint = MountPoint



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor = ...

class Fatquote(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    EPID_FIELD_NUMBER: builtins.int
    DCAP_FIELD_NUMBER: builtins.int
    NITRO_FIELD_NUMBER: builtins.int
    @property
    def epid(self) -> global___FatquoteEpid: ...
    @property
    def dcap(self) -> global___FatquoteDcap: ...
    @property
    def nitro(self) -> global___FatquoteNitro: ...
    def __init__(self,
        *,
        epid : typing.Optional[global___FatquoteEpid] = ...,
        dcap : typing.Optional[global___FatquoteDcap] = ...,
        nitro : typing.Optional[global___FatquoteNitro] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"dcap",b"dcap",u"epid",b"epid",u"fatquote",b"fatquote",u"nitro",b"nitro"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"dcap",b"dcap",u"epid",b"epid",u"fatquote",b"fatquote",u"nitro",b"nitro"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal[u"fatquote",b"fatquote"]) -> typing.Optional[typing_extensions.Literal["epid","dcap","nitro"]]: ...
global___Fatquote = Fatquote

class FatquoteEpid(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    IASRESPONSEBODY_FIELD_NUMBER: builtins.int
    IASCERTIFICATE_FIELD_NUMBER: builtins.int
    IASSIGNATURE_FIELD_NUMBER: builtins.int
    IASROOTCADER_FIELD_NUMBER: builtins.int
    iasResponseBody: builtins.bytes = ...
    iasCertificate: builtins.bytes = ...
    iasSignature: builtins.bytes = ...
    iasRootCaDer: builtins.bytes = ...
    def __init__(self,
        *,
        iasResponseBody : builtins.bytes = ...,
        iasCertificate : builtins.bytes = ...,
        iasSignature : builtins.bytes = ...,
        iasRootCaDer : builtins.bytes = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"iasCertificate",b"iasCertificate",u"iasResponseBody",b"iasResponseBody",u"iasRootCaDer",b"iasRootCaDer",u"iasSignature",b"iasSignature"]) -> None: ...
global___FatquoteEpid = FatquoteEpid

class FatquoteDcap(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    DCAPQUOTE_FIELD_NUMBER: builtins.int
    TCBINFO_FIELD_NUMBER: builtins.int
    QEIDENTITY_FIELD_NUMBER: builtins.int
    TCBSIGNCERT_FIELD_NUMBER: builtins.int
    QESIGNCERT_FIELD_NUMBER: builtins.int
    DCAPROOTCADER_FIELD_NUMBER: builtins.int
    dcapQuote: builtins.bytes = ...
    tcbInfo: builtins.bytes = ...
    qeIdentity: builtins.bytes = ...
    tcbSignCert: builtins.bytes = ...
    qeSignCert: builtins.bytes = ...
    dcapRootCaDer: builtins.bytes = ...
    def __init__(self,
        *,
        dcapQuote : builtins.bytes = ...,
        tcbInfo : builtins.bytes = ...,
        qeIdentity : builtins.bytes = ...,
        tcbSignCert : builtins.bytes = ...,
        qeSignCert : builtins.bytes = ...,
        dcapRootCaDer : builtins.bytes = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"dcapQuote",b"dcapQuote",u"dcapRootCaDer",b"dcapRootCaDer",u"qeIdentity",b"qeIdentity",u"qeSignCert",b"qeSignCert",u"tcbInfo",b"tcbInfo",u"tcbSignCert",b"tcbSignCert"]) -> None: ...
global___FatquoteDcap = FatquoteDcap

class FatquoteNitro(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    COSE_FIELD_NUMBER: builtins.int
    NITROROOTCADER_FIELD_NUMBER: builtins.int
    cose: builtins.bytes = ...
    nitroRootCaDer: builtins.bytes = ...
    def __init__(self,
        *,
        cose : builtins.bytes = ...,
        nitroRootCaDer : builtins.bytes = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"cose",b"cose",u"nitroRootCaDer",b"nitroRootCaDer"]) -> None: ...
global___FatquoteNitro = FatquoteNitro

class AttestationSpecification(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    INTELEPID_FIELD_NUMBER: builtins.int
    INTELDCAP_FIELD_NUMBER: builtins.int
    AWSNITRO_FIELD_NUMBER: builtins.int
    @property
    def intelEpid(self) -> global___AttestationSpecificationIntelEpid: ...
    @property
    def intelDcap(self) -> global___AttestationSpecificationIntelDcap: ...
    @property
    def awsNitro(self) -> global___AttestationSpecificationAwsNitro: ...
    def __init__(self,
        *,
        intelEpid : typing.Optional[global___AttestationSpecificationIntelEpid] = ...,
        intelDcap : typing.Optional[global___AttestationSpecificationIntelDcap] = ...,
        awsNitro : typing.Optional[global___AttestationSpecificationAwsNitro] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"attestation_specification",b"attestation_specification",u"awsNitro",b"awsNitro",u"intelDcap",b"intelDcap",u"intelEpid",b"intelEpid"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"attestation_specification",b"attestation_specification",u"awsNitro",b"awsNitro",u"intelDcap",b"intelDcap",u"intelEpid",b"intelEpid"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal[u"attestation_specification",b"attestation_specification"]) -> typing.Optional[typing_extensions.Literal["intelEpid","intelDcap","awsNitro"]]: ...
global___AttestationSpecification = AttestationSpecification

class AttestationSpecificationIntelEpid(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    MRENCLAVE_FIELD_NUMBER: builtins.int
    IASROOTCADER_FIELD_NUMBER: builtins.int
    ACCEPT_DEBUG_FIELD_NUMBER: builtins.int
    ACCEPT_GROUP_OUT_OF_DATE_FIELD_NUMBER: builtins.int
    ACCEPT_CONFIGURATION_NEEDED_FIELD_NUMBER: builtins.int
    mrenclave: builtins.bytes = ...
    iasRootCaDer: builtins.bytes = ...
    accept_debug: builtins.bool = ...
    accept_group_out_of_date: builtins.bool = ...
    accept_configuration_needed: builtins.bool = ...
    def __init__(self,
        *,
        mrenclave : builtins.bytes = ...,
        iasRootCaDer : builtins.bytes = ...,
        accept_debug : builtins.bool = ...,
        accept_group_out_of_date : builtins.bool = ...,
        accept_configuration_needed : builtins.bool = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"accept_configuration_needed",b"accept_configuration_needed",u"accept_debug",b"accept_debug",u"accept_group_out_of_date",b"accept_group_out_of_date",u"iasRootCaDer",b"iasRootCaDer",u"mrenclave",b"mrenclave"]) -> None: ...
global___AttestationSpecificationIntelEpid = AttestationSpecificationIntelEpid

class AttestationSpecificationIntelDcap(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    MRENCLAVE_FIELD_NUMBER: builtins.int
    DCAPROOTCADER_FIELD_NUMBER: builtins.int
    ACCEPT_DEBUG_FIELD_NUMBER: builtins.int
    ACCEPT_OUT_OF_DATE_FIELD_NUMBER: builtins.int
    ACCEPT_CONFIGURATION_NEEDED_FIELD_NUMBER: builtins.int
    ACCEPT_SW_HARDENING_NEEDED_FIELD_NUMBER: builtins.int
    ACCEPT_REVOKED_FIELD_NUMBER: builtins.int
    mrenclave: builtins.bytes = ...
    dcapRootCaDer: builtins.bytes = ...
    accept_debug: builtins.bool = ...
    accept_out_of_date: builtins.bool = ...
    accept_configuration_needed: builtins.bool = ...
    accept_sw_hardening_needed: builtins.bool = ...
    accept_revoked: builtins.bool = ...
    def __init__(self,
        *,
        mrenclave : builtins.bytes = ...,
        dcapRootCaDer : builtins.bytes = ...,
        accept_debug : builtins.bool = ...,
        accept_out_of_date : builtins.bool = ...,
        accept_configuration_needed : builtins.bool = ...,
        accept_sw_hardening_needed : builtins.bool = ...,
        accept_revoked : builtins.bool = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"accept_configuration_needed",b"accept_configuration_needed",u"accept_debug",b"accept_debug",u"accept_out_of_date",b"accept_out_of_date",u"accept_revoked",b"accept_revoked",u"accept_sw_hardening_needed",b"accept_sw_hardening_needed",u"dcapRootCaDer",b"dcapRootCaDer",u"mrenclave",b"mrenclave"]) -> None: ...
global___AttestationSpecificationIntelDcap = AttestationSpecificationIntelDcap

class AttestationSpecificationAwsNitro(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    NITROROOTCADER_FIELD_NUMBER: builtins.int
    PCRS_FIELD_NUMBER: builtins.int
    nitroRootCaDer: builtins.bytes = ...
    @property
    def pcrs(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.bytes]: ...
    def __init__(self,
        *,
        nitroRootCaDer : builtins.bytes = ...,
        pcrs : typing.Optional[typing.Iterable[builtins.bytes]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"nitroRootCaDer",b"nitroRootCaDer",u"pcrs",b"pcrs"]) -> None: ...
global___AttestationSpecificationAwsNitro = AttestationSpecificationAwsNitro
